# French lexemes common.py
lexemes = {
    "app_title": "Éditeur Notolog",
    "app_title_with_sub": "{app_title} - {sub_title}",

    "tree_filter_accessible_desc": "Champ de filtre de fichier",

    "menu_action_copy_file_path": "Copier le chemin du fichier",
    "menu_action_rename": "Renommer",
    "menu_action_delete": "Supprimer",
    "menu_action_delete_completely": "Supprimer définitivement",
    "menu_action_restore": "Restaurer",
    "menu_action_create_new_dir": "Créer un nouveau répertoire",

    "dialog_file_rename_title": "Renommer le fichier",
    "dialog_file_rename_field_label": "Entrer le nouveau nom du fichier",
    "dialog_file_rename_button_ok": "Renommer",
    "dialog_file_rename_warning_exists": "Un fichier portant le même nom existe déjà",

    "dialog_file_delete_title": "Supprimer le fichier",
    "dialog_file_delete_text": "Supprimer le fichier \"{file_name}\" ?",
    "dialog_file_delete_completely_title": "Supprimer définitivement le fichier",
    "dialog_file_delete_completely_text": "Supprimer définitivement le fichier \"{file_name}\" ?",
    "dialog_file_delete_error": "Impossible de supprimer le fichier, une erreur s'est produite",
    "dialog_file_delete_error_not_found": "Fichier non trouvé",

    "dialog_file_restore_title": "Restaurer le fichier",
    "dialog_file_restore_text": "Restaurer le fichier \"{file_name}\" ?",
    "dialog_file_restore_error": "Impossible de restaurer le fichier, une erreur s'est produite",
    "dialog_file_restore_warning_exists": "Un fichier portant le nom {file_name} existe déjà",

    "dialog_create_new_dir_title": "Créer un nouveau répertoire",
    "dialog_create_new_dir_label": "Nom du nouveau répertoire",
    "dialog_create_new_dir_input_placeholder_text": "Entrez le nom du répertoire",
    "dialog_create_new_dir_button_ok": "Créer",
    "dialog_create_new_dir_button_cancel": "Annuler",
    "dialog_create_new_dir_warning_empty_title": "Erreur de nom du nouveau répertoire",
    "dialog_create_new_dir_warning_empty_text": "Le nom du répertoire ne peut pas être vide",
    "dialog_create_new_dir_warning_too_long_title": "Erreur de nom du nouveau répertoire",
    "dialog_create_new_dir_warning_too_long_text": "Le nom du répertoire est trop long; maximum "
                                                   "{symbols} caractères autorisés !",
    "dialog_create_new_dir_error_existed": "Le répertoire existe déjà",
    "dialog_create_new_dir_error": "Impossible de créer le répertoire. Assurez-vous que le répertoire de destination "
                                   "{base_dir} est accessible en écriture",

    "dialog_message_box_title": "Message",
    "dialog_message_box_button_ok": "Fermer",

    "action_new_file_first_line_template_text": "Nouveau document",
    "action_open_file_dialog_caption": "Ouvrir un fichier",
    "action_save_as_file_dialog_caption": "Enregistrer un fichier",

    "dialog_save_empty_file_title": "Enregistrer un fichier vide",
    "dialog_save_empty_file_text": "Autoriser l'enregistrement du fichier sans contenu ?",

    "dialog_encrypt_file_title": "Chiffrer le fichier",
    "dialog_encrypt_file_text": "Chiffrer le fichier \"{file_name}\" ?",
    "encrypt_file_warning_file_is_already_encrypted": "Le fichier est déjà chiffré !",
    "dialog_encrypt_file_rewrite_existing_title": "Réécrire sur le fichier existant",
    "dialog_encrypt_file_rewrite_existing_text": "Réécrire sur le fichier existant \"{file_path}\" ?",

    "dialog_decrypt_file_title": "Déchiffrer le fichier",
    "dialog_decrypt_file_text": "Déchiffrer le fichier \"{file_name}\" ?",
    "decrypt_file_warning_file_is_not_encrypted": "Le fichier n'est pas chiffré !",
    "dialog_decrypt_file_rewrite_existing_title": "Réécrire sur le fichier existant",
    "dialog_decrypt_file_rewrite_existing_text": "Réécrire sur le fichier existant \"{file_path}\" ?",

    "dialog_encrypt_new_password_title": "Nouveau mot de passe",
    "dialog_encrypt_new_password_label": "Mot de passe :",
    "dialog_encrypt_new_password_input_placeholder_text": "Entrez un nouveau mot de passe",
    "dialog_encrypt_new_password_hint_label": "Indice :",
    "dialog_encrypt_new_password_hint_label_description":
        "L'indice n'est pas crypté et peut être lu dans le fichier ! "
        "\nÉvitez des indices évidents faciles à deviner, comme la date de naissance. "
        "\nEssayez d'utiliser une référence qui n'est pas facilement associable à vous.",
    "dialog_encrypt_new_password_hint_input_placeholder_text": "Entrez un indice (facultatif)",
    "dialog_encrypt_new_password_button_ok": "OK",
    "dialog_encrypt_new_password_button_cancel": "Annuler",
    "dialog_encrypt_new_password_warning_empty_title": "Avertissement",
    "dialog_encrypt_new_password_warning_empty_text": "Le champ du mot de passe ne peut pas être vide !",
    "dialog_encrypt_new_password_warning_too_long_title": "Avertissement",
    "dialog_encrypt_new_password_warning_too_long_text":
        "Le champ d'indice est trop long, {symbols} caractères maximum !",

    "dialog_encrypt_password_title": "Entrez le mot de passe",
    "dialog_encrypt_password_label": "Mot de passe :",
    "dialog_encrypt_password_input_placeholder_text": "Entrez le mot de passe",
    "dialog_encrypt_password_hint_label": "Indice :",
    "dialog_encrypt_password_button_ok": "OK",
    "dialog_encrypt_password_button_cancel": "Annuler",

    "dialog_encrypt_password_reset_title": "Réinitialiser le mot de passe de chiffrement",
    "dialog_encrypt_password_reset_text":
        "Êtes-vous sûr de vouloir réinitialiser le mot de passe de chiffrement actuel ?",
    "dialog_encrypt_password_reset_button_cancel": "Annuler",
    "dialog_encrypt_password_reset_button_yes": "Oui",

    "dialog_open_link_title": "Lien",
    "dialog_open_link_text": "Ouvrir le lien \"{url}\" dans un navigateur ?",

    "dialog_reset_settings_title": "Réinitialiser les paramètres ?",
    "dialog_reset_settings_text":
        "Toutes les données enregistrées dans les paramètres seront effacées, et l'application redémarrera pour "
        "appliquer les modifications.",

    "dialog_exit_unsaved_title": "Confirmer la Sortie",
    "dialog_exit_unsaved_text": "Le fichier ouvert '{file_name}' ne peut pas être enregistré. Continuer à quitter?",

    "message_app_config_file_access": "Accès refusé lors de l'accès au fichier de configuration de l'application à "
                                      "{file_path}. Définissez les autorisations correctes pour assurer un fonctionnement "
                                      "approprié.",

    "field_dir_path_dialog_caption": "Sélectionner un Répertoire",
    "field_file_path_dialog_caption": "Sélectionner un Fichier",

    "load_file_encryption_password_mismatch": "Non-concordance du mot de passe de chiffrement !",
    "load_file_encryption_password_incorrect": "Mot de passe de chiffrement incorrect !",
    "load_file_none_content_error": "Impossible de charger le fichier.",

    "action_new_file_error_occurred": "Impossible de créer le fichier ; une erreur est survenue."
                                      "\nVérifiez les autorisations du système de fichiers.",
    "save_active_file_error_occurred": "Impossible de sauvegarder le fichier ; une erreur est survenue.",

    "expandable_block_default_title": "Plus d'infos...",
    "expandable_block_open_close_tags_mismatch_warning": "Balises d'ouverture/fermeture du bloc <details> incompatibles",

    "dialog_color_picker_color_copied_to_the_clipboard": "Le texte formaté a été copié dans le presse-papiers",

    "popup_about_title": "Infos de l'Application",
    "popup_about_app_name_description": "Éditeur Markdown Python",

    "popup_about_version": "Version",
    "popup_about_license": "Licence",
    "popup_about_website": "Site Web",
    "popup_about_repository": "GitHub",
    "popup_about_pypi": "PyPi",
    "popup_about_date": "Date",

    "update_helper_new_version_is_available": "Une nouvelle version {latest_version} de l'application est disponible",
    "update_helper_latest_version_installed": "La dernière version de l'application est installée",

    "network_connection_error_empty": "Impossible d'obtenir des informations de réponse",
    "network_connection_error_connection_or_dns":
        "Hôte introuvable. Il pourrait y avoir un problème avec la connexion Internet ou le DNS.",
    "network_connection_error_connection_refused":
        "Connexion refusée. Le serveur pourrait être hors service ou il pourrait y avoir des problèmes de réseau.",
    "network_connection_error_connection_timed_out":
        "La connexion a expiré. Il pourrait y avoir des problèmes de réseau.",
    "network_connection_error_connection_404_error":
        "Erreur 404 de connexion. La page ou la ressource demandée n'est pas trouvée.",
    "network_connection_error_generic_with_status_code": "Échec de la requête avec le code d'état : {status_code}",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Part of the 'On-Device LLM' module.
- Functionality: Provides helper functions to initialize and manage ONNX Runtime sessions for supported LLMs.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

import os
import logging

from threading import Lock

# ONNX Runtime GenAI
from onnxruntime_genai import Generator, GeneratorParams, Model, Tokenizer, TokenizerStream


class ModelHelper:

    model: Model
    generator: Generator
    tokenizer: Tokenizer
    tokenizer_stream: TokenizerStream

    _instance = None  # Singleton instance
    _lock = Lock()

    # Set the max length to something sensible by default,
    # since otherwise it will be set to the entire context length
    search_options = {'max_length': 4096}

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            with cls._lock:
                if not cls._instance:
                    # Create the instance if it doesn't exist
                    cls._instance = super().__new__(cls)
        return cls._instance

    @classmethod
    def reload(cls, *args, **kwargs):
        """
        Reinitialize the singleton instance to allow controlled re-creation.
        """
        with cls._lock:
            # Create a new instance
            cls._instance = super().__new__(cls)

    def __init__(self, model_path: str, search_options: dict = None):
        # Prevent re-initialization if the instance is already set up.
        if hasattr(self, 'logger'):
            return

        # Use a lock to ensure initialization is thread-safe and atomic.
        with self._lock:
            # Double-check to prevent race conditions during initialization.
            if hasattr(self, 'logger'):
                return

            super(ModelHelper, self).__init__()

            self.logger = logging.getLogger('ondevice_llm_model_helper')

            # Check model path is correct
            self.model_path = model_path
            if (not os.path.isdir(self.model_path)
                    or not os.path.isfile(os.path.join(self.model_path, 'tokenizer.model'))):
                self.logger.warning(f'Model not found in {self.model_path}')
                self.model_path = None

            if search_options:
                # The list of the supported options
                supported_search_options = [
                    'do_sample', 'max_length', 'min_length', 'top_p', 'top_k', 'temperature', 'repetition_penalty']
                # Check and filter supported
                search_options = {key: value for key, value in search_options.items() if key in supported_search_options}
                # Convert the integer value of temperature to a float
                if 'temperature' in search_options and type(search_options['temperature']) is int:
                    search_options['temperature'] = self.convert_temperature(search_options['temperature'])
                # Add or rewrite params if passed
                self.search_options.update(search_options)

    def init_model(self):
        if hasattr(self, 'model') and self.model:
            return
        if self.model_path is None:
            raise AttributeError(f"'{type(self).__name__}' model path is not set")
        # Log event
        self.logger.info(f'Initializing model: {self.model_path}')
        # Init model
        self.model = Model(self.model_path)
        self.tokenizer = Tokenizer(self.model)
        self.tokenizer_stream = self.tokenizer.create_stream()

    def get_input_tokens(self, prompt):
        input_tokens = self.tokenizer.encode(prompt)
        return input_tokens

    def init_generator(self, input_tokens, search_options):
        # Configure your generator with necessary parameters
        params = GeneratorParams(self.model)
        vocab_size = getattr(params, 'vocab_size', 0)
        self.logger.debug(f'Model vocabulary size: {vocab_size}')
        params.set_search_options(**search_options)
        params.input_ids = input_tokens
        # params.use_cuda = False
        self.generator = Generator(self.model, params)  # type: Generator
        return self.generator

    def generate_output(self):
        if not self.generator.is_done():
            # Get tokens for this iteration
            self.generator.compute_logits()
            self.generator.generate_next_token()
            new_tokens = self.generator.get_next_tokens()
            # Only one token, double check token counting logic if changed
            new_token = new_tokens[0]
            outputs = self.tokenizer_stream.decode(new_token)
            # Or via decoding a batch
            # outputs = ''.join(self.tokenizer.decode_batch(new_tokens))
            self.logger.debug(f'Output: {outputs}: {new_token}')
            return outputs

    def get_model_name(self):
        # Safely truncate a long name
        return ModelHelper.truncate_string(os.path.basename(self.model_path))

    @staticmethod
    def truncate_string(text, max_text_length=32):
        if len(text) <= max_text_length:
            return text

        # Calculating the space to be allocated to the start and end of the string around the ellipsis
        part_length = (max_text_length - 3) // 2
        start_part = text[:part_length]
        end_part = text[-part_length:]

        # Create the truncated string with an ellipsis in the middle
        return f"{start_part}...{end_part}"

    @staticmethod
    def convert_temperature(temperature: int = 0):
        """ Convert the integer value of temperature to a float. """
        return temperature / 100



# Greek lexemes color_picker_dialog.py
lexemes = {
    "color_picker_title": "Επιλογή Χρώματος",

    "color_picker_color_black": "Μαύρο",
    "color_picker_color_silver": "Ασημί",
    "color_picker_color_gray": "Γκρι",
    "color_picker_color_white": "Λευκό",
    "color_picker_color_maroon": "Καστανοκόκκινο",
    "color_picker_color_red": "Κόκκινο",
    "color_picker_color_purple": "Μωβ",
    "color_picker_color_fuchsia": "Φούξια",
    "color_picker_color_green": "Πράσινο",
    "color_picker_color_lime": "Λαχανί",
    "color_picker_color_olive": "Ελαιόλαδο",
    "color_picker_color_yellow": "Κίτρινο",
    "color_picker_color_navy": "Μπλε ναυτικό",
    "color_picker_color_blue": "Μπλε",
    "color_picker_color_teal": "Γαλάζιο",
    "color_picker_color_aqua": "Υδρόμπλε",
    "color_picker_color_aliceblue": "Αλίς μπλε",
    "color_picker_color_antiquewhite": "Αντίκα Λευκό",
    "color_picker_color_aquamarine": "Ακουαμαρίν",
    "color_picker_color_azure": "Γαλανό",
    "color_picker_color_beige": "Μπεζ",
    "color_picker_color_bisque": "Μπισκ",
    "color_picker_color_blanchedalmond": "Αμύγδαλο ξεφλουδισμένο",
    "color_picker_color_blueviolet": "Μπλε Μωβ",
    "color_picker_color_brown": "Καφέ",
    "color_picker_color_burlywood": "Ξύλο Βέργα",
    "color_picker_color_cadetblue": "Μπλε Στρατιώτη",
    "color_picker_color_chartreuse": "Σαρτρέζ",
    "color_picker_color_chocolate": "Σοκολάτα",
    "color_picker_color_coral": "Κοραλί",
    "color_picker_color_cornflowerblue": "Μπλε Κενταυρίδας",
    "color_picker_color_cornsilk": "Μετάξι Καλαμποκιού",
    "color_picker_color_crimson": "Κόκκινο Βαθύ",
    "color_picker_color_cyan": "Κυανό",
    "color_picker_color_darkblue": "Σκούρο Μπλε",
    "color_picker_color_darkcyan": "Σκούρο Κυανό",
    "color_picker_color_darkgoldenrod": "Σκούρο Χρυσαφί Ράβδος",
    "color_picker_color_darkgray": "Σκούρο Γκρι",
    "color_picker_color_darkgreen": "Σκούρο Πράσινο",
    "color_picker_color_darkkhaki": "Σκούρο Χακί",
    "color_picker_color_darkmagenta": "Σκούρο Ματζέντα",
    "color_picker_color_darkolivegreen": "Σκούρο Πράσινο Ελιάς",
    "color_picker_color_darkorange": "Σκούρο Πορτοκαλί",
    "color_picker_color_darkorchid": "Σκούρο Ορχιδέα",
    "color_picker_color_darkred": "Σκούρο Κόκκινο",
    "color_picker_color_darksalmon": "Σκούρο Σολομός",
    "color_picker_color_darkseagreen": "Σκούρο Θαλασσί Πράσινο",
    "color_picker_color_darkslateblue": "Σκούρο Σχιστόλιθος Μπλε",
    "color_picker_color_darkslategray": "Σκούρο Σχιστόλιθος Γκρι",
    "color_picker_color_darkturquoise": "Σκούρο Τυρκουάζ",
    "color_picker_color_darkviolet": "Σκούρο Μωβ",
    "color_picker_color_deeppink": "Βαθύ Ροζ",
    "color_picker_color_deepskyblue": "Βαθύ Ουρανί Μπλε",
    "color_picker_color_dimgray": "Θαμπό Γκρι",
    "color_picker_color_dodgerblue": "Διαφυγή Μπλε",
    "color_picker_color_firebrick": "Τούβλο Φωτιάς",
    "color_picker_color_floralwhite": "Φλοράλ Λευκό",
    "color_picker_color_forestgreen": "Πράσινο Δάσους",
    "color_picker_color_gainsboro": "Γκέινσμπορο",
    "color_picker_color_ghostwhite": "Φάντασμα Λευκό",
    "color_picker_color_gold": "Χρυσαφί",
    "color_picker_color_goldenrod": "Χρυσαφί Ράβδος",
    "color_picker_color_greenyellow": "Πράσινο Κίτρινο",
    "color_picker_color_honeydew": "Δροσοσταλίδα",
    "color_picker_color_hotpink": "Ζεστό Ροζ",
    "color_picker_color_indianred": "Ινδικό Κόκκινο",
    "color_picker_color_indigo": "Ινδικό",
    "color_picker_color_ivory": "Ιβουάρ",
    "color_picker_color_khaki": "Χακί",
    "color_picker_color_lavender": "Λεβάντα",
    "color_picker_color_lavenderblush": "Λεβάντα Ροζ",
    "color_picker_color_lawngreen": "Πράσινο Γκαζόν",
    "color_picker_color_lemonchiffon": "Λεμονί Σιφόν",
    "color_picker_color_lightblue": "Ανοιχτό Μπλε",
    "color_picker_color_lightcoral": "Ανοιχτό Κοραλί",
    "color_picker_color_lightcyan": "Ανοιχτό Κυανό",
    "color_picker_color_lightgoldenrodyellow": "Ανοιχτό Χρυσαφί Ράβδος Κίτρινο",
    "color_picker_color_lightgray": "Ανοιχτό Γκρι",
    "color_picker_color_lightgreen": "Ανοιχτό Πράσινο",
    "color_picker_color_lightpink": "Ανοιχτό Ροζ",
    "color_picker_color_lightsalmon": "Ανοιχτό Σολομός",
    "color_picker_color_lightseagreen": "Ανοιχτό Θαλασσί Πράσινο",
    "color_picker_color_lightskyblue": "Ανοιχτό Ουρανί Μπλε",
    "color_picker_color_lightslategray": "Ανοιχτό Σχιστόλιθος Γκρι",
    "color_picker_color_lightsteelblue": "Ανοιχτό Μέταλλο Μπλε",
    "color_picker_color_lightyellow": "Ανοιχτό Κίτρινο",
    "color_picker_color_limegreen": "Λαχανί Πράσινο",
    "color_picker_color_linen": "Λινάρι",
    "color_picker_color_magenta": "Ματζέντα",
    "color_picker_color_mediumaquamarine": "Μέτριο Ακουαμαρίν",
    "color_picker_color_mediumblue": "Μέτριο Μπλε",
    "color_picker_color_mediumorchid": "Μέτριο Ορχιδέα",
    "color_picker_color_mediumpurple": "Μέτριο Μωβ",
    "color_picker_color_mediumseagreen": "Μέτριο Θαλασσί Πράσινο",
    "color_picker_color_mediumslateblue": "Μέτριο Σχιστόλιθος Μπλε",
    "color_picker_color_mediumspringgreen": "Μέτριο Ανοιξιάτικο Πράσινο",
    "color_picker_color_mediumturquoise": "Μέτριο Τυρκουάζ",
    "color_picker_color_mediumvioletred": "Μέτριο Μωβ Κόκκινο",
    "color_picker_color_midnightblue": "Μεσάνυχτα Μπλε",
    "color_picker_color_mintcream": "Κρέμα Μέντας",
    "color_picker_color_mistyrose": "Θολό Ροζ",
    "color_picker_color_moccasin": "Μοκασίνι",
    "color_picker_color_navajowhite": "Ναβάχο Λευκό",
    "color_picker_color_oldlace": "Παλιά Δαντέλα",
    "color_picker_color_olivedrab": "Στρατιωτικό Πράσινο",
    "color_picker_color_orange": "Πορτοκαλί",
    "color_picker_color_orangered": "Πορτοκαλοκόκκινο",
    "color_picker_color_orchid": "Ορχιδέα",
    "color_picker_color_palegoldenrod": "Αχνό Χρυσαφί Ράβδος",
    "color_picker_color_palegreen": "Αχνό Πράσινο",
    "color_picker_color_paleturquoise": "Αχνό Τυρκουάζ",
    "color_picker_color_palevioletred": "Αχνό Μωβ Κόκκινο",
    "color_picker_color_papayawhip": "Παπάγια Χτυπητή",
    "color_picker_color_peachpuff": "Ροδάκινο Φουσκωτό",
    "color_picker_color_peru": "Περού",
    "color_picker_color_pink": "Ροζ",
    "color_picker_color_plum": "Δαμάσκηνο",
    "color_picker_color_powderblue": "Πούδρα Μπλε",
    "color_picker_color_rebeccapurple": "Ρεβέκκα Μωβ",
    "color_picker_color_rosybrown": "Ροδαλό Καφέ",
    "color_picker_color_royalblue": "Βασιλικό Μπλε",
    "color_picker_color_saddlebrown": "Καφέ Σέλα",
    "color_picker_color_salmon": "Σολομός",
    "color_picker_color_sandybrown": "Αμμώδες Καφέ",
    "color_picker_color_seagreen": "Θαλασσί Πράσινο",
    "color_picker_color_seashell": "Κοχύλι",
    "color_picker_color_sienna": "Σιέννα",
    "color_picker_color_skyblue": "Ουρανί Μπλε",
    "color_picker_color_slateblue": "Σχιστόλιθος Μπλε",
    "color_picker_color_slategray": "Σχιστόλιθος Γκρι",
    "color_picker_color_snow": "Χιόνι",
    "color_picker_color_springgreen": "Ανοιξιάτικο Πράσινο",
    "color_picker_color_steelblue": "Μέταλλο Μπλε",
    "color_picker_color_tan": "Ταν",
    "color_picker_color_thistle": "Γαϊδουράγκαθο",
    "color_picker_color_tomato": "Ντομάτα",
    "color_picker_color_turquoise": "Τυρκουάζ",
    "color_picker_color_violet": "Βιολετί",
    "color_picker_color_wheat": "Σιτάρι",
    "color_picker_color_whitesmoke": "Λευκός Καπνός",
    "color_picker_color_yellowgreen": "Κίτρινο Πράσινο",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Contains unit and integration tests for the related functionality.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from notolog.app_config import AppConfig
from notolog.ui.ai_assistant.ai_assistant import EnumMessageType

from .. import is_module_available

# Explicitly check if the module is available to avoid importing non-existent libraries.
if is_module_available('openai_api'):
    from notolog.modules.openai_api.prompt_manager import PromptManager
else:
    from PySide6.QtCore import QObject as PromptManager

import pytest
import logging


class TestPromptManager:

    @pytest.fixture(scope="function", autouse=True)
    def test_obj_app_config(self, mocker):
        # Mock AppConfig's get_logger_level method to suppress logging during tests.
        mocker.patch.object(AppConfig, 'get_logger_level', return_value=logging.NOTSET)

        # Create an AppConfig instance and set it to test mode.
        _app_config = AppConfig()
        _app_config.set_test_mode(True)

        yield _app_config

    @pytest.fixture(scope="function")
    def test_prompt_manager(self, request):
        """
        Testing object fixture.
        May contain minimal logic for testing purposes.
        """

        # Retrieve parameter values from the test request.
        system_prompt, max_history_size, reload = request.param if hasattr(request, 'param') else (None, None, None)

        if reload:
            # Reload as new instance to avoid param caching
            PromptManager.reload()

        yield PromptManager(system_prompt=system_prompt, max_history_size=max_history_size, parent=None)

    @pytest.fixture(scope="function")
    def test_params_fixture(self, request):
        # Retrieve parameter values from the test request.
        param_values = request.param

        yield param_values

    @pytest.fixture(scope="function")
    def test_exp_params_fixture(self, request):
        # Retrieve parameter values from the test request.
        param_values = request.param

        yield param_values

    @pytest.mark.parametrize(
        "test_prompt_manager, test_exp_params_fixture",
        [
            (('prompt1', None, False), ('**system**:\nprompt1', [{'role': 'system', 'content': 'prompt1'}])),
            ((None, None, False), ('**system**:\nprompt1', [{'role': 'system', 'content': 'prompt1'}])),
            (('prompt2', None, True), ('**system**:\nprompt2', [{'role': 'system', 'content': 'prompt2'}])),
            ((None, None, True), ('', [])),
        ],
        indirect=True
    )
    def test_init_and_get_prompt_history(self, test_prompt_manager: PromptManager, test_exp_params_fixture):

        exp_history_result, exp_prompt_result = test_exp_params_fixture

        history_result = test_prompt_manager.get_history()

        assert history_result == exp_history_result

        prompt_result = test_prompt_manager.get_prompt()

        assert prompt_result == exp_prompt_result

    @pytest.mark.parametrize(
        "test_prompt_manager, test_params_fixture, test_exp_params_fixture",
        [
            ((None, None, None), (None, None), None),
            ((None, None, None), (None, 'content0'), None),
            ((None, None, None), ('role1', 'content0'), None),
            ((None, None, None), ('system', 'content1'), {'content': 'content1', 'role': 'system'}),
            ((None, None, None), ('user', 'content2'), {'content': 'content2', 'role': 'user'}),
            ((None, None, None), ('assistant', 'content3'), {'content': 'content3', 'role': 'assistant'}),
        ],
        indirect=True
    )
    def test_get_prompt_message(self, test_prompt_manager: PromptManager,
                                test_params_fixture, test_exp_params_fixture):
        role, content = test_params_fixture
        exp_result = test_exp_params_fixture

        result = test_prompt_manager.get_prompt_message(role, content)

        assert result == exp_result

    @pytest.mark.parametrize(
        "test_prompt_manager, test_params_fixture, test_exp_params_fixture",
        [
            # Resetting the history
            (('prompt0', None, True), ('', 0, 0, EnumMessageType.USER_INPUT),
             '**system**:\nprompt0\n\n**user**:\n'),
            (('prompt1', None, True), ('smthU1', 0, 0, EnumMessageType.USER_INPUT),
             '**system**:\nprompt1\n\n**user**:\nsmthU1'),
            (('prompt2', None, True), ('smthU2', 123, 456, EnumMessageType.USER_INPUT),
             '**system**:\nprompt2\n\n**user**:\nsmthU2'),
            (('prompt3', None, True), ('smthA3', 0, 0, EnumMessageType.RESPONSE),
             '**system**:\nprompt3\n\n**assistant**:\nsmthA3'),
            (('prompt4', None, True), ('smthU4', 0, 0, EnumMessageType.USER_INPUT),
             '**system**:\nprompt4\n\n**user**:\nsmthU4'),
            # Remembering the history (by msg_id), but with a zero limit
            (('prompt111', 0, True), ('smthU111', 0, 0, EnumMessageType.USER_INPUT),
             '**system**:\nprompt111\n\n**user**:\nsmthU111'),
            (('promptXYZ', None, False), ('smthA222', 0, 1, EnumMessageType.RESPONSE),
             '**system**:\nprompt111\n\n**user**:\nsmthU111\n\n**assistant**:\nsmthA222'),
            # Remembering the history (by msg_id), but with a limit
            (('prompt111', 3, True), ('smthU111', 0, 0, EnumMessageType.USER_INPUT),
             '**system**:\nprompt111\n\n**user**:\nsmthU111'),
            (('promptXYZ', None, False), ('smthA222', 0, 1, EnumMessageType.RESPONSE),
             '**system**:\nprompt111\n\n**user**:\nsmthU111\n\n**assistant**:\nsmthA222'),
            ((None, None, False), ('smthU333', 2, 0, EnumMessageType.USER_INPUT),
             '**system**:\nprompt111\n\n**user**:\nsmthU111\n\n**assistant**:\nsmthA222\n\n**user**:\nsmthU333'),
            ((None, None, False), ('smthA444', 0, 4, EnumMessageType.RESPONSE),
             '**system**:\nprompt111\n\n**assistant**:\nsmthA222\n\n**user**:\nsmthU333\n\n**assistant**:\nsmthA444'),
            ((None, None, False), ('smthA123', 0, 3, EnumMessageType.RESPONSE),
             '**system**:\nprompt111\n\n**user**:\nsmthU333\n\n**assistant**:\nsmthA444\n\n**assistant**:\nsmthA123'),
        ],
        indirect=True
    )
    def test_add_message(self, mocker, test_prompt_manager: PromptManager,
                         test_params_fixture, test_exp_params_fixture):
        message_text, request_msg_id, response_msg_id, message_type = test_params_fixture
        exp_result = test_exp_params_fixture

        # Retrieve callback methods for verification.
        mock_add_request = mocker.patch.object(test_prompt_manager, 'add_request', wraps=test_prompt_manager.add_request)
        mock_add_response = mocker.patch.object(test_prompt_manager, 'add_response', wraps=test_prompt_manager.add_response)

        test_prompt_manager.add_message(message_text, request_msg_id, response_msg_id, message_type)

        if message_type == EnumMessageType.USER_INPUT:
            mock_add_request.assert_called_once()
            # Check the parameters passed to the mocked function(s)
            assert str(mock_add_request.call_args) == "call('%s', %d)" % (message_text, request_msg_id)
        elif message_type == EnumMessageType.RESPONSE:
            mock_add_response.assert_called_once()
            # Check the parameters passed to the mocked function(s)
            assert (str(mock_add_response.call_args) == "call('%s', %d, %d)"
                    % (message_text, response_msg_id, request_msg_id))
        else:
            mock_add_request.assert_not_called()
            mock_add_response.assert_not_called()

        result = test_prompt_manager.get_history()

        assert result == exp_result

    @pytest.mark.parametrize(
        "test_prompt_manager, test_params_fixture, test_exp_params_fixture",
        [
            ((None, None, None), ('', 0), {'role': 'system', 'content': ''}),
            ((None, None, None), ('smth1', 1), {'role': 'system', 'content': 'smth1'}),
        ],
        indirect=True
    )
    def test_add_system(self, mocker, test_prompt_manager: PromptManager,
                        test_params_fixture, test_exp_params_fixture):
        system_instruct, msg_id = test_params_fixture
        exp_call_param = test_exp_params_fixture

        # Retrieve callback methods for verification.
        mock_update_or_add_entry = mocker.patch.object(test_prompt_manager, 'update_or_add_entry',
                                                       wraps=test_prompt_manager.update_or_add_entry)

        test_prompt_manager.add_system(system_instruct, msg_id)

        mock_update_or_add_entry.assert_called_once()
        # Check the parameters passed to the mocked function(s)
        assert (str(mock_update_or_add_entry.call_args) == "call(msg_id=%d, data=%s)" % (msg_id, exp_call_param))

    @pytest.mark.parametrize(
        "test_prompt_manager, test_params_fixture, test_exp_params_fixture",
        [
            ((None, None, None), ('', 0), {'role': 'user', 'content': ''}),
            ((None, None, None), ('smth1', 1), {'role': 'user', 'content': 'smth1'}),
        ],
        indirect=True
    )
    def test_add_request(self, mocker, test_prompt_manager: PromptManager,
                         test_params_fixture, test_exp_params_fixture):
        request_text, request_msg_id = test_params_fixture
        exp_call_param = test_exp_params_fixture

        # Retrieve callback methods for verification.
        mock_update_or_add_entry = mocker.patch.object(test_prompt_manager, 'update_or_add_entry',
                                                       wraps=test_prompt_manager.update_or_add_entry)

        test_prompt_manager.add_request(request_text, request_msg_id)

        mock_update_or_add_entry.assert_called_once()
        # Check the parameters passed to the mocked function(s)
        assert (str(mock_update_or_add_entry.call_args) == "call(msg_id=%d, data=%s)" % (request_msg_id, exp_call_param))

    @pytest.mark.parametrize(
        "test_prompt_manager, test_params_fixture, test_exp_params_fixture",
        [
            ((None, None, None), ('', 0, None), {'role': 'assistant', 'content': ''}),
            ((None, None, None), ('smth1', 1, None), {'role': 'assistant', 'content': 'smth1'}),
        ],
        indirect=True
    )
    def test_add_response(self, mocker, test_prompt_manager: PromptManager,
                          test_params_fixture, test_exp_params_fixture):
        response_text, response_msg_id, request_msg_id = test_params_fixture
        exp_call_param = test_exp_params_fixture

        # Retrieve callback methods for verification.
        mock_update_or_add_entry = mocker.patch.object(test_prompt_manager, 'update_or_add_entry',
                                                       wraps=test_prompt_manager.update_or_add_entry)

        test_prompt_manager.add_response(response_text, response_msg_id, request_msg_id)

        mock_update_or_add_entry.assert_called_once()
        # Check the parameters passed to the mocked function(s)
        assert (str(mock_update_or_add_entry.call_args) == "call(msg_id=%d, data=%s)" % (response_msg_id, exp_call_param))

    @pytest.mark.parametrize(
        "test_prompt_manager, test_params_fixture, test_exp_params_fixture",
        [
            ((None, None, True), (0, '', True),
             ([], [{'id': 0, 'data': ''}])),
            ((None, None, True), (1, '', True),
             ([], [{'id': 1, 'data': ''}])),
            ((None, None, False), (1, '', False),
             ([{'id': 1, 'data': ''}], [{'id': 1, 'data': ''}])),
            ((None, None, False), (2, 'smth2', True),
             ([{'id': 1, 'data': ''}], [{'id': 1, 'data': ''}, {'id': 2, 'data': 'smth2'}])),
            ((None, None, False), (3, 'smth3', True),
             ([{'id': 1, 'data': ''}, {'id': 2, 'data': 'smth2'}],
              [{'id': 1, 'data': ''}, {'id': 2, 'data': 'smth2'}, {'id': 3, 'data': 'smth3'}])),
        ],
        indirect=True
    )
    def test_update_or_add_entry(self, mocker, test_prompt_manager: PromptManager,
                                 test_params_fixture, test_exp_params_fixture):
        msg_id, data, limit_history_size = test_params_fixture
        exp_prev_history_result, exp_history_result = test_exp_params_fixture

        # Retrieve callback methods for verification.
        mock_limit_history_size = mocker.patch.object(test_prompt_manager, 'limit_history_size')

        assert test_prompt_manager.history == exp_prev_history_result

        test_prompt_manager.update_or_add_entry(msg_id, data)

        assert test_prompt_manager.history == exp_history_result

        if limit_history_size:
            mock_limit_history_size.assert_called_once()
        else:
            mock_limit_history_size.assert_not_called()

    @pytest.mark.parametrize(
        "test_prompt_manager, test_params_fixture, test_exp_params_fixture",
        [
            # Mocking the history after reload
            ((None, None, True), ([
                {'id': 1, 'data': {'content': 'system1', 'role': 'system'}}], ''),
             None),
            ((None, None, None), ([
                {'id': 1, 'data': {'content': 'system1', 'role': 'system'}}], 'system'),
             {'content': 'system1', 'role': 'system'}),
            ((None, None, None), ([
                {'id': 1, 'data': {'content': 'system1', 'role': 'system'}},
                {'id': 2, 'data': {'content': 'user1', 'role': 'user'}},
                {'id': 3, 'data': {'content': 'assistant1', 'role': 'assistant'}}], 'system'),
             {'content': 'system1', 'role': 'system'}),
            ((None, None, None), ([
                {'id': 1, 'data': {'content': 'system1', 'role': 'system'}},
                {'id': 2, 'data': {'content': 'user1', 'role': 'user'}},
                {'id': 3, 'data': {'content': 'assistant1', 'role': 'assistant'}}], 'user'),
             {'content': 'user1', 'role': 'user'}),
            ((None, None, None), ([
                {'id': 1, 'data': {'content': 'system1', 'role': 'system'}},
                {'id': 2, 'data': {'content': 'user1', 'role': 'user'}},
                {'id': 3, 'data': {'content': 'assistant1', 'role': 'assistant'}}], 'assistant'),
             {'content': 'assistant1', 'role': 'assistant'}),
            ((None, None, None), ([
                {'id': 1, 'data': {'content': 'system1', 'role': 'system'}},
                {'id': 2, 'data': {'content': 'user1', 'role': 'user'}},
                {'id': 3, 'data': {'content': 'assistant1', 'role': 'assistant'}},
                {'id': 4, 'data': {'content': 'user2', 'role': 'user'}}], 'user'),
             {'content': 'user2', 'role': 'user'}),
            ((None, None, None), ([
                {'id': 1, 'data': {'content': 'system1', 'role': 'system'}},
                {'id': 4, 'data': {'content': 'user1', 'role': 'user'}},
                {'id': 3, 'data': {'content': 'assistant1', 'role': 'assistant'}},
                {'id': 2, 'data': {'content': 'user2', 'role': 'user'}}], 'user'),
             {'content': 'user2', 'role': 'user'}),
        ],
        indirect=True
    )
    def test_find_last_message_by_role(self, test_prompt_manager: PromptManager,
                                       test_params_fixture, test_exp_params_fixture):
        history, role = test_params_fixture
        exp_result = test_exp_params_fixture

        setattr(test_prompt_manager, 'history', history)

        result = test_prompt_manager.find_last_message_by_role(role)

        assert result == exp_result



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Contains unit and integration tests for the related functionality.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from notolog.app_package import AppPackage
from notolog.app_config import AppConfig

import pytest

import os
import tomli
import shutil
import logging


class TestAppConfig:

    @pytest.fixture(scope="function")
    def test_obj_app_package(self, mocker, request):
        get_config = request.param

        # Mock methods
        mocker.patch.object(AppPackage, 'get_config', return_value=get_config)
        mocker.patch.object(AppPackage, 'set_config', return_value=None)
        mocker.patch.object(AppPackage, 'set_type', return_value=None)

        # Init object
        _app_package = AppPackage()

        # Mock get package type results
        mocker.patch.object(_app_package, 'get_type', wraps=_app_package.get_type)

        yield _app_package

    @pytest.fixture(scope="function")
    def test_obj_app_config(self, mocker, request):
        app_config, skip_validation = request.param if isinstance(request.param, tuple) else (request.param, False)

        if skip_validation:
            # Allow updates without validating complete configuration.
            mocker.patch.object(AppConfig, 'validate_config', return_value=True)
        mocker.patch.object(AppConfig, 'load_config', return_value=app_config)

        # Mock libs results
        mocker.patch.object(tomli, 'loads', wraps=tomli.loads)
        mocker.patch.object(shutil, 'copy', return_value=None)
        # mocker.patch.object(os, 'remove', return_value=None)

        _app_config = AppConfig()
        _app_config.set_test_mode(True)
        _app_config.load_initial_conf()

        yield _app_config

    @pytest.fixture(scope="function")
    def test_exp_params_fixture(self, request):
        # Retrieve parameter values from the test request.
        param_values = request.param

        yield param_values

    @pytest.mark.parametrize(
        "test_obj_app_package, test_obj_app_config, test_exp_params_fixture",
        [
            (({}), dict({'package': {'type': ''}, 'font': {'base_size': 13}}),
             ('notolog_app_config_qa.toml', 'pip', False)),
            (({}), dict({'package': {'type': 'smth1'}, 'font': {'base_size': 13}}),
             ('notolog_app_config_qa.toml', 'pip', False)),
            (({'package': {'type': 'bin'}}), dict({'package': {'type': 'smth1'}, 'font': {'base_size': 13}}),
             ('notolog_app_config_qa_bin.toml', 'bin', True)),
            (({'package': {'type': 'bin'}}), dict({'package': {'type': 'bin'}, 'font': {'base_size': 13}}),
             ('notolog_app_config_qa_bin.toml', 'bin', True)),
            (({'package': {'type': 'bin'}}), dict({'package': {'type': 'pip'}, 'font': {'base_size': 13}}),
             ('notolog_app_config_qa_bin.toml', 'bin', True)),
        ],
        indirect=True
    )
    def test_general_checks(self, mocker, test_obj_app_package: AppPackage, test_obj_app_config: AppConfig,
                            test_exp_params_fixture):

        exp_file_path, exp_package_type, exp_result = test_exp_params_fixture

        mock_app_config_update_handler = mocker.patch.object(test_obj_app_config, 'app_config_update_handler',
                                                             return_value=None)
        mock_app_config_logger_warning = mocker.patch.object(test_obj_app_config.logger, 'warning',
                                                             return_value=None)

        mock_app_package_set_config = mocker.patch.object(test_obj_app_package, 'set_config', return_value=None)
        mock_app_package_set = mocker.patch.object(test_obj_app_package, 'set_type', return_value=None)

        expected_app_config = test_obj_app_config.app_config

        test_obj_app_config.load_initial_conf()

        # Match package file and config results
        assert (test_obj_app_config.get_package_type() == test_obj_app_package.get_type()) == exp_result

        assert test_obj_app_config.get_package_type() == exp_package_type
        assert test_obj_app_config.get_font_base_size() == test_obj_app_config.app_config['font']['base_size']

        assert os.path.basename(test_obj_app_config.get_app_config_path()) == os.path.basename(exp_file_path)
        assert test_obj_app_config.app_config == expected_app_config

        # The config is forced with mocked data; the below approach will not work.
        # assert test_obj_app_config.toml_file_path == test_obj_app_config.get_app_config_path()

        if test_obj_app_config.toml_file_path != test_obj_app_config.get_app_config_path():
            assert mock_app_config_logger_warning.call_count == 1
            assert test_obj_app_config.app_config['package']['type'] != test_obj_app_package.get_type()

        mock_app_config_update_handler.assert_not_called()
        mock_app_package_set_config.assert_not_called()
        mock_app_package_set.assert_not_called()

    @pytest.mark.parametrize(
        "test_obj_app_package, test_obj_app_config, test_exp_params_fixture",
        [
            (({}), {'package': {'type': None}, 'font': {'base_size': None}}, ('', 'pip')),
            (({}), {'package': {'type': None}, 'font': {'base_size': None}}, ('test1', 'pip')),
            (({}), {'package': {'type': None}, 'font': {'base_size': None}}, ('bin', 'bin')),
            (({}), {'package': {'type': 'pip'}, 'font': {'base_size': None}}, ('bin', 'bin')),
            (({'package': {'type': 'pip'}}), {'package': {'type': 'smth1'}, 'font': {'base_size': None}}, ('bin', 'bin')),
            (({}), {'package': {'type': None}, 'font': {'base_size': None}}, ('', 'pip')),
        ],
        indirect=True
    )
    def test_setup_package(self, mocker, test_obj_app_package: AppPackage, test_obj_app_config: AppConfig,
                           test_exp_params_fixture):

        setup_type, exp_result = test_exp_params_fixture

        mock_app_config_update_handler = mocker.patch.object(test_obj_app_config, 'app_config_update_handler',
                                                             return_value=None)
        mock_app_config_load_initial_conf = mocker.patch.object(test_obj_app_config, 'load_initial_conf',
                                                                return_value=None)
        mock_app_config_save_app_config = mocker.patch.object(test_obj_app_config, 'save_app_config',
                                                              return_value=None)

        mock_app_package_set_config = mocker.patch.object(test_obj_app_package, 'set_config', return_value=None)
        mock_app_package_set = mocker.patch.object(test_obj_app_package, 'set_type', return_value=None)

        previously_set_package = test_obj_app_config.get_package_type()

        test_obj_app_config.setup_package(setup_type)

        mock_app_config_load_initial_conf.assert_called_once()

        mock_app_config_save_app_config.assert_called_once()

        if not previously_set_package:
            mock_app_config_update_handler.assert_called_once_with({'package_type': exp_result})

        if test_obj_app_package.get_type() != exp_result:
            mock_app_package_set.assert_called_once_with(exp_result)
            assert str(mock_app_package_set.call_args) == "call('%s')" % exp_result
        else:
            mock_app_package_set_config.assert_not_called()
            mock_app_package_set.assert_not_called()

    @pytest.mark.parametrize(
        "test_obj_app_package, test_obj_app_config, test_exp_params_fixture",
        [
            (
                {'package': {'type': 'smth1'}}, {},
                ({'package': {'type': 'smth2'}, 'font': {'base_size': None}}, False, False)
            ),
            (
                {'package': {'type': 'smth1'}}, {},
                ({'package': {'type': 'pip'}, 'font': {'base_size': None}}, False, False)
            ),
            (
                    {'package': {'type': 'bin'}}, {},
                    ({'package': {'type': 'smth1'}, 'font': {'base_size': None}}, True, False)
            ),
            (
                {'package': {'type': 'bin'}}, {},
                ({'package': {'type': 'pip'}, 'font': {'base_size': None}}, True, False)
            ),
            (
                {'package': {'type': 'smth1'}}, {},
                (AppConfig().get_default_app_config(), False, True)
            ),
            (
                {'package': {'type': 'bin'}}, {},
                (AppConfig().get_default_app_config(), True, True)
            ),

        ],
        indirect=True
    )
    def test_validate_configs(self, test_obj_app_package: AppPackage, test_obj_app_config: AppConfig,
                              test_exp_params_fixture):

        app_config_to_check, exp_app_package_result, exp_app_config_result = test_exp_params_fixture

        app_package_config = test_obj_app_package.get_config()

        assert test_obj_app_package.validate_config(app_package_config) == exp_app_package_result
        assert test_obj_app_config.validate_config(app_config_to_check) == exp_app_config_result

        assert 'package' in app_package_config
        assert 'type' in app_package_config['package']

        assert test_obj_app_package.validate_type(test_obj_app_package.get_type()) == exp_app_package_result

    @pytest.mark.parametrize(
        "test_obj_app_package, test_obj_app_config",
        [
            # This creates a separate file for each package value
            ({'package': {'type': 'pip'}}, AppConfig().get_config()),
            ({'package': {'type': 'bin'}}, AppConfig().get_config()),
            # Restore the real app config to reset pytest object state
            ({'package': {'type': AppPackage.default_package}}, AppConfig().get_config()),
        ],
        indirect=True
    )
    def test_finish(self, test_obj_app_package: AppPackage, test_obj_app_config: AppConfig):
        # Verify the test file's suffix
        app_config_path = test_obj_app_config.get_app_config_path()
        assert '_qa' in app_config_path
        # Deleting the config file and ensures it was existed
        assert test_obj_app_config.delete_app_config()
        assert not os.path.exists(app_config_path)

    @pytest.mark.parametrize(
        "test_obj_app_package, test_obj_app_config, test_exp_params_fixture",
        [
            (({}), ({'logger': {'level': ''}}, True), logging.INFO),
            (({}), ({'logger': {'level': 'notset'}}, True), logging.NOTSET),
            (({}), ({'logger': {'level': 'info'}}, True), logging.INFO),
            (({}), ({'logger': {'level': 'DEBUG'}}, True), logging.DEBUG),
            (({}), ({'logger': {'level': 'cRitICaL'}}, True), logging.CRITICAL),
        ],
        indirect=True
    )
    def test_get_logger_level(self, test_obj_app_package: AppPackage, test_obj_app_config: AppConfig,
                              test_exp_params_fixture):
        exp_logger_level_result = test_exp_params_fixture

        logger_level_result = test_obj_app_config.get_logger_level()

        assert logger_level_result == exp_logger_level_result



# Greek lexemes settings_dialog.py
lexemes = {
    "tab_openai_api_config": "OpenAI API",

    "module_openai_api_label": "OpenAI API",
    "module_openai_api_url_label": "URL του API",
    "module_openai_api_url_input_placeholder_text": "URL του API",
    "module_openai_api_url_input_accessible_description":
        "Το URL του API της OpenAI είναι η διεύθυνση του τελικού σημείου του API, το οποίο μπορεί να διαφέρει\n"
        "ανάλογα με την υπηρεσία και την έκδοση. Ο Βοηθός AI χρησιμοποιεί αυτό για συνομιλίες ή συμπληρώσεις κειμένου.\n"
        "Ανατρέξτε στην επίσημη τεκμηρίωση του API της OpenAI για να αποκτήσετε το τρέχον URL.",
    "module_openai_api_key_label": "Κλειδί API",
    "module_openai_api_key_input_placeholder_text": "Κλειδί API",
    "module_openai_api_key_input_accessible_description":
        "Το κλειδί API της OpenAI είναι ένα μυστικό διακριτικό που χρησιμοποιείται για τον έλεγχο ταυτότητας αιτημάτων\n"
        "στο τελικό σημείο του API.",
    "module_openai_api_supported_models_label": "Υποστηριζόμενα Μοντέλα",
    "module_openai_api_model_names_combo_placeholder_text": "Επιλέξτε ένα Μοντέλο",
    "module_openai_api_model_names_combo_accessible_description":
        "Επιλέξτε από τα υποστηριζόμενα μοντέλα για συζητήσεις συνομιλίας.",

    "module_openai_api_base_system_prompt_label": "Σύστημα Προτροπής",
    "module_openai_api_base_system_prompt_edit_placeholder_text":
        "Βασική προτροπή συστήματος που προηγείται κάθε αιτήματος",
    "module_openai_api_base_system_prompt_edit_accessible_description":
        "Βασική προτροπή συστήματος που προηγείται κάθε αιτήματος.\n"
        "Συνήθως απλό κείμενο με οδηγίες ή χαρακτηριστικά ρόλου.",

    "module_openai_api_base_response_temperature_label": "Θερμοκρασία: {temperature}",
    "module_openai_api_base_response_temperature_input_accessible_description":
        "Ρυθμίζει την τυχαιότητα των αποκρίσεων του μοντέλου. Υψηλότερες τιμές παράγουν πιο ποικίλες εξόδους, "
        "ενώ χαμηλότερες τιμές καθιστούν τις αποκρίσεις πιο προβλέψιμες.",

    "module_openai_api_base_response_max_tokens_label": "Μέγιστος αριθμός tokens ανά απόκριση",
    "module_openai_api_base_response_max_tokens_input_accessible_description":
        "Ο μέγιστος αριθμός tokens που λαμβάνονται σε μια απόκριση, όπως λέξεις και στίξη, "
        "διαχειρίζοντας το μήκος της εξόδου.",

    "module_openai_api_config_prompt_history_size_label": "Μέγεθος Ιστορικού Προτροπών",
    "module_openai_api_config_prompt_history_size_input_accessible_description":
        "Ελέγχει τον αριθμό των καταχωρήσεων στο ιστορικό προτροπών που διατηρεί το σύστημα για αναφορά.\n"
        "Μια τιμή μηδέν επιτρέπει απεριόριστες καταχωρήσεις."
}



# English lexemes settings_dialog.py
lexemes = {
    "tab_ondevice_llm_config": "On Device LLM",

    "module_ondevice_llm_config_label": "On Device LLM Model",
    "module_ondevice_llm_config_path_label": "ONNX Model Location",
    "module_ondevice_llm_config_path_input_placeholder_text": "Path to the Model Directory",
    "module_ondevice_llm_config_path_input_accessible_description":
        "An input field with a selector to specify the path to the model directory where ONNX files are located.\n"
        "Supported models are in the ONNX format, which stands for Open Neural Network Exchange, an open standard\n"
        "format for machine learning models.",

    "module_ondevice_llm_config_response_temperature_label": "Temperature: {temperature}",
    "module_ondevice_llm_config_response_temperature_input_accessible_description":
        "Adjusts the randomness of the model's responses. Higher values produce more varied outputs,\n"
        "while lower values make responses more predictable.",

    "module_ondevice_llm_config_response_max_tokens_label": "Maximum Response Tokens",
    "module_ondevice_llm_config_response_max_tokens_input_accessible_description":
        "Sets the maximum number of tokens to be received in a response, such as words and punctuation,\n"
        "controlling the length of the output.",

    "module_ondevice_llm_config_prompt_history_size_label": "Prompt History Size",
    "module_ondevice_llm_config_prompt_history_size_input_accessible_description":
        "Controls the number of entries in the prompt history that the system retains for reference.\n"
        "A zero value allows for unlimited entries."
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Provides app toolbar UI.
- Functionality: Displays the app's toolbar icons and search form. Supports context menu for adjusting icon elements.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import QSize
from PySide6.QtGui import QAction, QColor
from PySide6.QtWidgets import QToolBar, QWidget, QMenu, QToolButton, QSizePolicy

from . import Settings
from . import Lexemes
from . import ThemeHelper

from ..ui.search_form import SearchForm

from typing import TYPE_CHECKING

import logging

if TYPE_CHECKING:
    from typing import Union  # noqa
    from ..notolog_editor import NotologEditor  # noqa


class ToolBar(QToolBar):
    # Main window toolbar class

    def __init__(self, parent, actions=None, refresh=None):
        """
        Args:
            parent (optional): Parent object
            actions (List[Dict[str, Any]], optional): The map with the toolbar's icons
            refresh (Callable[[int, int], int], optional): A lambda function that refreshes a toolbar
        """
        super(ToolBar, self).__init__(parent)

        self.parent = parent  # type: NotologEditor

        if self.parent and hasattr(self.parent, 'font'):
            # Apply font from the main window to the dialog
            self.setFont(self.parent.font())

        self.logger = logging.getLogger('toolbar')

        self.actions = actions if actions else {}
        self.refresh = refresh

        self.settings = Settings()

        self.theme_helper = ThemeHelper()

        # Load lexemes for selected language and scope
        self.lexemes = Lexemes(self.settings.app_language, default_scope='toolbar')

        if self.settings.toolbar_icons is None or self.settings.toolbar_icons == 0:
            """
            Default bit-mask is 131070 when all icons are checked.
            """
            self.settings.toolbar_icons = 131070
            self.logger.debug('Starting with default icons mask "%d"' % self.settings.toolbar_icons)

        self.search_form = None  # type: Union[SearchForm, QWidget, None]

        self.toolbar_save_button = None  # type: Union[QToolButton, None]
        self.toolbar_edit_button = None  # type: Union[QToolButton, None]

        self.setMovable(False)

        self.init_ui()

    def init_ui(self):
        """
        Build the toolbar's UI components by dynamically creating toolbar icons and adding a search form
        based on defined actions and settings.
        """

        # Adjust layout margins for proper spacing
        self.setContentsMargins(0, 0, 5, 2)

        # Initialize previous icon type to manage delimiters.
        prev_type = None

        # Iterate over action configurations.
        for icon in self.actions:
            if icon['type'] == 'action':
                # Skip the icon addition if it's not enabled in the settings.
                if not (self.settings.toolbar_icons & pow(2, icon['weight'])):
                    continue
                # Skip if the icon's active state check function returns False.
                if ('active_state_check' in icon
                        and callable(icon['active_state_check'])
                        and not icon['active_state_check']()):
                    continue
                # Add the toolbar icon if all conditions are met.
                self.append_toolbar_icon(icon)
            # Add a separator unless the last added item was also a delimiter.
            elif icon['type'] == 'delimiter' and prev_type != 'delimiter':
                self.addSeparator()
            # Update previous icon type to manage delimiters correctly.
            prev_type = icon['type']

        # Add a spacer to separate icons from the search form.
        central_spacer = QWidget(self)
        central_spacer.setSizePolicy(QSizePolicy.Policy.MinimumExpanding, QSizePolicy.Policy.MinimumExpanding)
        self.addWidget(central_spacer)

        # Initialize and add the search form.
        self.search_form = SearchForm(parent=self)
        self.addWidget(self.search_form)

        # Set the toolbar's stylesheet from the theme helper.
        self.setStyleSheet(self.theme_helper.get_css('toolbar'))

    def append_toolbar_icon(self, conf):
        """
        Helper to create, add to the toolbar and return button with an icon.
        """
        # Theme icon with a fallback to a system one
        system_icon = conf['system_icon'] if 'system_icon' in conf else None
        theme_icon = conf['theme_icon'] if 'theme_icon' in conf else None
        theme_icon_color = QColor(conf['color']) if 'color' in conf \
            else self.theme_helper.get_color('toolbar_icon_color_default')
        icon = self.theme_helper.get_icon(theme_icon=theme_icon, system_icon=system_icon, color=theme_icon_color)

        # Button's action
        label = conf['label'] if 'label' in conf else ''
        icon_action = QAction(icon, label, self)
        action = conf['action'] if 'action' in conf else None  # Action triggered on click
        if action is not None:
            icon_action.triggered.connect(action)

        # Toolbar button itself
        icon_button = QToolButton(self)
        if 'name' in conf:
            icon_button.setObjectName(conf['name'])
        icon_button.setToolTip(label)
        # Set the button height to match the search input field height, maintaining the aspect ratio.
        icon_width = icon_height = int(icon_button.height() * 0.8)
        icon_button.setIconSize(QSize(icon_width, icon_height))
        icon_button.setDefaultAction(icon_action)
        # icon_action.setChecked(True)
        if 'accessible_name' in conf:
            icon_button.setAccessibleName(conf['accessible_name'])

        # Add the button to the toolbar
        self.addWidget(icon_button)

        # Add internal variable to access the icon later, say for state toggle
        if 'var_name' in conf:
            if hasattr(self, conf['var_name']):
                self.logger.debug('Variable "%s" is already set! Re-writing it...' % conf['var_name'])
            setattr(self, conf['var_name'], icon_button)  # type: QToolButton
        # If the icon has a switched off check, check it here
        if ('switched_off_check' in conf
                and callable(conf['switched_off_check'])
                and conf['switched_off_check']()):
            # Switch the icon off
            icon_button.setDisabled(True)

    def contextMenuEvent(self, event):
        """
        Render context menu event upon the toolbar right mouse click
        """
        current_action = self.actionAt(event.pos())
        if current_action is None:
            return

        if self.actions is None:
            return

        context_menu = QMenu(self)

        _weights = 0
        for index, label in enumerate(self.actions, 1):
            if 'type' not in label or label['type'] != 'action':
                context_menu.addSeparator()
                continue

            # Get settings weight of the item
            settings_weight = pow(2, label['weight'])

            if _weights & settings_weight:
                # The item is already added or the active state item that shouldn't be duplicated
                continue

            button = QAction(label['label'], self)
            button.setFont(self.font())

            # Method def instead of lambda
            def slot(checked, i=label['weight']):
                self.toolbar_menu_item(checked, i)

            # Check button.toggled.connect()
            button.triggered[bool].connect(slot)
            button.setCheckable(True)
            # Check the item is in the settings weights
            button.setChecked(self.settings.toolbar_icons & settings_weight)

            context_menu.addAction(button)

            # Collect items already added to the context menu
            _weights |= settings_weight

        # PoC remove element from the toolbar
        # delete_action  = menu.addAction("Hide")

        context_menu.exec(event.globalPos())

        # PoC Remove element from the toolbar
        # if action == delete_action:
        #    self.removeAction(current_action)

    def toolbar_menu_item(self, checked: bool, index: int) -> None:
        """
        Draw context menu item with a checkbox

        Args:
            checked (bool): Checked or not
            index (int): Index in a toolbar menu mapping
        """
        pi = pow(2, index)
        self.logger.debug('checked:{} index:{} pi:{}' . format(checked, index, pi))
        if checked:
            self.settings.toolbar_icons |= pi
        else:
            self.settings.toolbar_icons ^= pi
        # Re-draw toolbar with a callback
        if callable(self.refresh):
            self.refresh()

    # PoC Remove element from the toolbar
    # def removeAction(self, action):
    #    """
    #    Override removeAction() method upon QWidgetAction
    #
    #    Args:
    #        action (QAction):
    #    """
    #    super(ToolBar, self).removeAction(action)
    #
    #    self.logger.debug('Remove element action "%s"' % action)



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Lexemes manager.
- Functionality: Handles the app's supported languages and lexeme scopes, including modules.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

import os
import logging
import importlib.util

from typing import Union, Dict

from ..enums.languages import Languages


class Lexemes:
    def __init__(self, language: str = 'en', default_scope: str = 'common', lexemes_dir: str = None):
        if not language or language in ('.', '..'):
            # Default language ('en')
            language = str(Languages.default())

        self.language = language
        self._default_scope = default_scope
        self._lexemes_dir = lexemes_dir

        # Lexeme storage
        self.lexemes = {}

        self.logger = logging.getLogger('lexemes')

        self.lexemes = self.load_lexemes()

    def get_lexemes_dir(self):
        # Check if lexemes dir is re-written
        lexemes_dir = self._lexemes_dir if self._lexemes_dir else os.path.dirname(__file__)
        # Lexemes dir for the selected language
        lexemes_lang_dir = os.path.join(lexemes_dir, self.language)
        if not os.path.isdir(lexemes_lang_dir):
            # Default language ('en')
            lexemes_lang_dir = os.path.join(lexemes_dir, str(Languages.default()))
        return lexemes_lang_dir

    def load_lexemes(self) -> Dict[str, Dict]:
        # Get the directory path of lexemes relative to this file
        lexemes_lang_dir = self.get_lexemes_dir()

        lexeme_files = []
        if os.path.isdir(lexemes_lang_dir):
            for file_name in os.listdir(lexemes_lang_dir):
                # Load all .py files from the dir excluding __init__.py file if exists
                if file_name.endswith('.py') and file_name != '__init__.py':
                    lexeme_files.append(os.path.join(lexemes_lang_dir, file_name))

        # Import and collect lexemes from each lexeme file
        lexemes = {}
        for file_path in lexeme_files:
            scope_name = os.path.splitext(os.path.basename(file_path))[0]
            if scope_name not in lexemes:
                lexemes[scope_name] = {}
            spec = importlib.util.spec_from_file_location(scope_name, file_path)
            lexemes_module = importlib.util.module_from_spec(spec)
            spec.loader.exec_module(lexemes_module)
            if hasattr(lexemes_module, 'lexemes'):
                lexemes[scope_name].update(lexemes_module.lexemes)

        return lexemes

    def set_language(self, language: str) -> None:
        """
        Set language and re-load all corresponded lexemes
        @param language: string key of the language to load
        @return: None
        """
        self.language = language
        self.load_lexemes()

    def get(self, name, scope: str = None, **kwargs) -> Union[str, None]:
        """
        Get lexeme by its name and scope (optional; default 'common')
        @param name: lexeme's name key
        @param scope: lexeme's scope, e.g. common, encrypt, etc. If not set default_scope is in use.
        @return: either string value or None if lexeme's scope is not found
        """
        if not scope:
            # Use this method to get default scope as it may contain extra logic
            scope = self.get_default_scope()

        lexeme = self.lexemes[scope].get(name, '') if scope in self.lexemes else None
        if isinstance(lexeme, bytes):
            lexeme = lexeme.decode('utf-8')

        if kwargs:
            # Process placeholders if set, e.g. name="abc" in a string like "... {name} ..." become "... abc ..."
            try:
                lexeme = lexeme.format(**kwargs)
            except AttributeError:
                self.logger.warning('Lexeme "%s" not found' % name)
            except KeyError:
                self.logger.warning('Lexeme "%s" params [%s] not found for replacement'
                                    % (lexeme, ', '.join(kwargs.keys())))
        return lexeme

    def get_all(self):
        # Return all lexemes set
        return self.lexemes

    def get_by_scope(self, scope):
        return self.lexemes[scope] if scope in self.lexemes else None

    def get_default_scope(self):
        # Default scope if set
        scope = self._default_scope

        # If scope is not found in loaded lexemes
        if scope not in self.lexemes:
            # The very last fallback option
            scope = 'common'

        return scope

    def get_full_key(self, name, scope: str = None):
        if not scope:
            # Use this method to get default scope as it may contain extra logic
            scope = self.get_default_scope()

        return f"{scope}_{name}"



# Latin lexemes main_menu.py
lexemes = {
    "group_file_label": "Fasciculus",
    "group_edit_label": "Recensere",
    "group_tools_label": "Instrumenta",
    "group_help_label": "Auxilium",

    "actions_file_label_new_document": "Documentum novum",
    "actions_file_accessible_name_new_document": "Documentum novum",
    "actions_file_label_open": "Aperire",
    "actions_file_accessible_name_open": "Documentum aperire",
    "actions_file_label_save": "Servare",
    "actions_file_accessible_name_save": "Documentum Servare",
    "actions_file_label_save_as": "Salvare ut",
    "actions_file_accessible_name_save_as": "Salvare ut exemplar documenti",
    "actions_file_label_settings": "Configurationes",
    "actions_file_accessible_name_settings": "Configurationes Applicatio",
    "actions_file_label_reset_settings": "Reddere ad initium",
    "actions_file_accessible_name_reset_settings": "Reddere ad initium",
    "actions_file_label_exit": "Excedere",
    "actions_file_accessible_name_exit": "Excedere applicationem",

    "actions_edit_edit_mode": "Recensere",
    "actions_edit_accessible_name_edit_mode": "Recensere fasciculum",
    "actions_edit_source_mode": "Fons",
    "actions_edit_accessible_name_source_mode": "Codex fontis",
    "actions_edit_bold": "Audax",
    "actions_edit_accessible_name_bold": "Formatum textus audacis",
    "actions_edit_italic": "Italicum",
    "actions_edit_accessible_name_italic": "Formatum textus italicum",
    "actions_edit_underline": "Sublineare",
    "actions_edit_accessible_name_underline": "Formatum textus sublinearis",
    "actions_edit_strikethrough": "Expungere",
    "actions_edit_accessible_name_strikethrough": "Formatum textus expungendi",
    "actions_edit_blockquote": "Blockquote",
    "actions_edit_accessible_name_blockquote": "Formatum textus blockquote",

    "actions_tools_ai_assistant": "Assistens AI",
    "actions_tools_accessible_name_ai_assistant": "Initiare Assistens AI",
    "actions_tools_color_picker": "Selector Coloris",
    "actions_tools_accessible_name_color_picker": "Eligere colorem textus",

    "actions_help_label_md_syntax": "Syntaxis markdown",
    "actions_help_accessible_name_md_syntax": "Syntaxis markdown",
    "actions_help_label_check_for_updates": "Repaginas scrutare",
    "actions_help_accessible_name_check_for_updates": "Repaginas scrutare",
    "actions_help_label_bug_report": "Falla nuntiare",
    "actions_help_accessible_name_bug_report": "Falla nuntiare",
    "actions_help_label_about": "De",
    "actions_help_accessible_name_about": "De",
}



# Italian lexemes toolbar.py
lexemes = {
    "actions_label_new_document": "Nuovo documento",
    "actions_accessible_name_new_document": "Nuovo documento",
    "actions_label_open": "Apri",
    "actions_accessible_name_open": "Apri documento",
    "actions_label_save": "Salva",
    "actions_accessible_name_save": "Salva documento",
    "actions_label_save_as": "Salva come",
    "actions_accessible_name_save_as": "Salva come copia del documento",
    "actions_label_sync": "Sincronizza",
    "actions_accessible_name_sync": "Sincronizza documento con storage esterno",
    "actions_label_edit": "Modifica",
    "actions_accessible_name_edit": "Modifica file",
    "actions_label_source": "Sorgente",
    "actions_accessible_name_source": "Codice sorgente",
    "actions_label_encrypt": "Cripta",
    "actions_accessible_name_encrypt": "Cripta file",
    "actions_label_decrypt": "Decripta",
    "actions_accessible_name_decrypt": "Decripta file",
    "actions_label_bold": "Grassetto",
    "actions_accessible_name_bold": "Formatta testo in grassetto",
    "actions_label_italic": "Corsivo",
    "actions_accessible_name_italic": "Formatta testo in corsivo",
    "actions_label_underline": "Sottolineato",
    "actions_accessible_name_underline": "Formatta testo sottolineato",
    "actions_label_strikethrough": "Barrato",
    "actions_accessible_name_strikethrough": "Formatta testo barrato",
    "actions_label_blockquote": "Citazione",
    "actions_accessible_name_blockquote": "Formatta testo come citazione",
    "actions_label_color_picker": "Selettore di colore",
    "actions_accessible_name_color_picker": "Scegli un colore di testo",
    "actions_label_ai_assistant": "Assistente IA",
    "actions_accessible_name_ai_assistant": "Esegui assistente IA",
    "actions_label_about": "Informazioni",
    "actions_accessible_name_about": "Informazioni sull'App",
    "actions_label_exit": "Esci",
    "actions_accessible_name_exit": "Esci dall'App",
    "actions_label_settings": "Impostazioni",
    "actions_accessible_name_settings": "Impostazioni dell'app",

    "search_input_label": "Cerca",
    "search_input_placeholder_text": "Cerca testo nel documento",
    "search_input_accessible_description": "Campo di input per la ricerca",
    "search_buttons_label_clear": "Pulisci ricerca",
    "search_buttons_accessible_name_clear": "Pulisci campo di ricerca",
    "search_buttons_label_prev": "Ricerca precedente",
    "search_buttons_accessible_name_prev": "Risultato ricerca precedente",
    "search_buttons_label_next": "Ricerca successiva",
    "search_buttons_accessible_name_next": "Risultato ricerca successiva",
    "search_case_sensitive_accessible_name": "Distingui maiuscole e minuscole",
    "search_case_sensitive_tooltip": "Ricerca con distinzione tra maiuscole e minuscole",
}



# Korean lexemes settings_dialog.py
lexemes = {
    "tab_ondevice_llm_config": "기기 내 LLM",

    "module_ondevice_llm_config_label": "기기 내 LLM 모델",
    "module_ondevice_llm_config_path_label": "ONNX 모델 위치",
    "module_ondevice_llm_config_path_input_placeholder_text": "모델 디렉토리 경로",
    "module_ondevice_llm_config_path_input_accessible_description":
        "ONNX 파일이 위치한 모델 디렉토리 경로를 지정하기 위한 선택기가 있는 입력 필드입니다.\n"
        "지원되는 모델은 ONNX 형식으로, Open Neural Network Exchange의 약자이며 기계 학습 모델의 개방형 표준\n"
        "형식입니다.",

    "module_ondevice_llm_config_response_temperature_label": "온도: {temperature}",
    "module_ondevice_llm_config_response_temperature_input_accessible_description":
        "모델의 반응의 무작위성을 조절합니다. 높은 값은 더 다양한 출력을 생성하며,\n"
        "낮은 값은 반응을 더 예측 가능하게 합니다.",

    "module_ondevice_llm_config_response_max_tokens_label": "최대 응답 토큰 수",
    "module_ondevice_llm_config_response_max_tokens_input_accessible_description":
        "응답에서 받을 수 있는 토큰의 최대 수를 설정합니다. 여기에는 단어와 구두점이 포함되며,\n"
        "출력의 길이를 제어합니다.",

    "module_ondevice_llm_config_prompt_history_size_label": "프롬프트 기록 크기",
    "module_ondevice_llm_config_prompt_history_size_input_accessible_description":
        "시스템에 의해 참조용으로 유지되는 프롬프트 기록의 항목 수를 제어합니다.\n"
        "0값은 무제한 항목을 허용합니다."
}



# Georgian lexemes settings_dialog.py
lexemes = {
    "tab_ondevice_llm_config": "მოწყობილობაზე LLM",

    "module_ondevice_llm_config_label": "მოწყობილობაზე LLM მოდელი",
    "module_ondevice_llm_config_path_label": "ONNX მოდელის მდებარეობა",
    "module_ondevice_llm_config_path_input_placeholder_text": "მოდელის დირექტორიის გზა",
    "module_ondevice_llm_config_path_input_accessible_description":
        "შეყვანის ველი სელექტორით მოდელის დირექტორიის გზის მითითებისთვის, სადაც ONNX ფაილები მდებარეობენ.\n"
        "მხარდაჭერილი მოდელები არის ONNX ფორმატში, რაც აღნიშნავს ოფლიანური ნეირონული ქსელის გაცვლის ღია სტანდარტს\n"
        "მანქანური სწავლის მოდელების ფორმატისთვის.",

    "module_ondevice_llm_config_response_temperature_label": "ტემპერატურა: {temperature}",
    "module_ondevice_llm_config_response_temperature_input_accessible_description":
        "რეგულირებს მოდელის პასუხების შემთხვევითობას. მაღალი მნიშვნელობები ქმნიან უფრო ვარიაციულ გამოსავალებს,\n"
        "დაბალი მნიშვნელობები გამოიღებენ უფრო წინასწარ განსაზღვრავ პასუხებს.",

    "module_ondevice_llm_config_response_max_tokens_label": "პასუხში მაქსიმუმ ტოკენების რაოდენობა",
    "module_ondevice_llm_config_response_max_tokens_input_accessible_description":
        "შეზღუდავს მოდელის პასუხებში ტოკენების რაოდენობას აქტუალური კონტექსტის ფანჯრის ლიმიტამდე.\n"
        "ნულოვანი მნიშვნელობა გამოიყენებს კონტექსტის ფანჯრის ჩათვლითი ტევადობას.",

    "module_ondevice_llm_config_prompt_history_size_label": "პრომპტის ისტორიის ზომა",
    "module_ondevice_llm_config_prompt_history_size_input_accessible_description":
        "კონტროლირებს პრომპტების ისტორიაში ჩანაწერების რაოდენობას, რომელიც სისტემამ სარეფერენციოდ ინახავს.\n"
        "ნულოვანი მნიშვნელობა აძლევს ულიმიტო შეტანებას."
}



# Japanese lexemes main_menu.py
lexemes = {
    "group_file_label": "ファイル",
    "group_edit_label": "編集",
    "group_tools_label": "ツール",
    "group_help_label": "ヘルプ",

    "actions_file_label_new_document": "新しいドキュメント",
    "actions_file_accessible_name_new_document": "新しいドキュメント",
    "actions_file_label_open": "開く",
    "actions_file_accessible_name_open": "ドキュメントを開く",
    "actions_file_label_save": "保存",
    "actions_file_accessible_name_save": "ドキュメントを保存",
    "actions_file_label_save_as": "名前を付けて保存",
    "actions_file_accessible_name_save_as": "ドキュメントのコピーを保存",
    "actions_file_label_settings": "設定",
    "actions_file_accessible_name_settings": "アプリ設定",
    "actions_file_label_reset_settings": "設定をリセット",
    "actions_file_accessible_name_reset_settings": "設定をリセット",
    "actions_file_label_exit": "アプリを終了",
    "actions_file_accessible_name_exit": "アプリを終了",

    "actions_edit_edit_mode": "編集",
    "actions_edit_accessible_name_edit_mode": "ファイルを編集",
    "actions_edit_source_mode": "ソース",
    "actions_edit_accessible_name_source_mode": "ソースコード",
    "actions_edit_bold": "太字",
    "actions_edit_accessible_name_bold": "太字テキスト形式",
    "actions_edit_italic": "イタリック",
    "actions_edit_accessible_name_italic": "イタリックテキスト形式",
    "actions_edit_underline": "下線",
    "actions_edit_accessible_name_underline": "下線テキスト形式",
    "actions_edit_strikethrough": "取り消し線",
    "actions_edit_accessible_name_strikethrough": "取り消し線テキスト形式",
    "actions_edit_blockquote": "ブロック引用",
    "actions_edit_accessible_name_blockquote": "ブロック引用テキスト形式",

    "actions_tools_ai_assistant": "AIアシスタント",
    "actions_tools_accessible_name_ai_assistant": "AIアシスタントを起動",
    "actions_tools_color_picker": "カラーピッカー",
    "actions_tools_accessible_name_color_picker": "テキスト色を選択",

    "actions_help_label_md_syntax": "Markdown 構文",
    "actions_help_accessible_name_md_syntax": "Markdown 構文",
    "actions_help_label_check_for_updates": "更新を確認",
    "actions_help_accessible_name_check_for_updates": "更新を確認",
    "actions_help_label_bug_report": "バグ報告",
    "actions_help_accessible_name_bug_report": "バグ報告を送信",
    "actions_help_label_about": "情報",
    "actions_help_accessible_name_about": "情報",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Contains unit and integration tests for the related functionality.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from notolog.app_config import AppConfig
from notolog.ui.ai_assistant.ai_assistant import EnumMessageType

from .. import is_module_available

# Explicitly check if the module is available to avoid importing non-existent libraries.
if is_module_available('ondevice_llm'):
    from notolog.modules.ondevice_llm.prompt_manager import PromptManager
else:
    from PySide6.QtCore import QObject as PromptManager

import pytest
import logging


class TestPromptManager:

    @pytest.fixture(scope="function", autouse=True)
    def test_obj_app_config(self, mocker):
        # Mock AppConfig's get_logger_level method to suppress logging during tests.
        mocker.patch.object(AppConfig, 'get_logger_level', return_value=logging.NOTSET)

        # Create an AppConfig instance and set it to test mode.
        _app_config = AppConfig()
        _app_config.set_test_mode(True)

        yield _app_config

    @pytest.fixture(scope="function")
    def test_prompt_manager(self, request):
        """
        Testing object fixture.
        May contain minimal logic for testing purposes.
        """

        # Retrieve parameter values from the test request.
        system_prompt, max_history_size, reload = request.param if hasattr(request, 'param') else (None, None, None)

        if reload:
            # Reload the singleton instance to ensure a fresh start for each test.
            PromptManager.reload()

        yield PromptManager(system_prompt=system_prompt, max_history_size=max_history_size, parent=None)

    @pytest.fixture(scope="function")
    def test_params_fixture(self, request):
        # Retrieve parameter values from the test request.
        param_values = request.param

        yield param_values

    @pytest.fixture(scope="function")
    def test_exp_params_fixture(self, request):
        # Retrieve parameter values from the test request.
        param_values = request.param

        yield param_values

    @pytest.mark.parametrize(
        "test_prompt_manager, test_exp_params_fixture",
        [
            (('prompt1', None, False), ('**system**:\nprompt1', '<|endoftext|><|system|>\nprompt1<|end|>\n')),
            ((None, None, False), ('**system**:\nprompt1', '<|endoftext|><|system|>\nprompt1<|end|>\n')),
            (('prompt2', None, True), ('**system**:\nprompt2', '<|endoftext|><|system|>\nprompt2<|end|>\n')),
            ((None, None, True), ('', '<|endoftext|>')),
        ],
        indirect=True
    )
    def test_init_and_get_prompt_history(self, test_prompt_manager: PromptManager, test_exp_params_fixture):
        # Test initialization and prompt history retrieval.
        exp_history_result, exp_prompt_result = test_exp_params_fixture

        history_result = test_prompt_manager.get_history()

        assert history_result == exp_history_result

        prompt_result = test_prompt_manager.get_prompt()

        assert prompt_result == exp_prompt_result

    @pytest.mark.parametrize(
        "test_prompt_manager, test_params_fixture, test_exp_params_fixture",
        [
            # Define test cases for getting a single prompt message with expected results.
            ((None, None, None), None, '<|user|>\nNone<|end|>\n<|assistant|>\n'),
            ((None, None, None), 'content0', '<|user|>\ncontent0<|end|>\n<|assistant|>\n'),
            ((None, None, None), 'content1', '<|user|>\ncontent1<|end|>\n<|assistant|>\n'),
        ],
        indirect=True
    )
    def test_get_prompt_message(self, test_prompt_manager: PromptManager,
                                test_params_fixture, test_exp_params_fixture):
        content = test_params_fixture
        exp_result = test_exp_params_fixture

        result = test_prompt_manager.get_prompt_message(str(content))

        assert result == exp_result

    @pytest.mark.parametrize(
        "test_prompt_manager, test_params_fixture, test_exp_params_fixture",
        [
            # Resetting the history
            (('prompt0', None, True), ('', 0, 0, EnumMessageType.USER_INPUT),
             '**system**:\nprompt0\n\n**user**:'),
            (('prompt1', None, True), ('smthU1', 0, 0, EnumMessageType.USER_INPUT),
             '**system**:\nprompt1\n\n**user**:\nsmthU1'),
            (('prompt2', None, True), ('smthU2', 123, 456, EnumMessageType.USER_INPUT),
             '**system**:\nprompt2\n\n**user**:\nsmthU2'),
            (('prompt3', None, True), ('smthA3', 0, 0, EnumMessageType.RESPONSE),
             '**system**:\nprompt3'),  # Not added as no match found
            (('prompt4', None, True), ('smthU4', 0, 0, EnumMessageType.USER_INPUT),
             '**system**:\nprompt4\n\n**user**:\nsmthU4'),
            # Remembering the history (by msg_id), but with a zero limit
            (('prompt111', 0, True), ('smthU111', 0, 0, EnumMessageType.USER_INPUT),
             '**system**:\nprompt111\n\n**user**:\nsmthU111'),
            (('promptXYZ', None, False), ('smthA222', 0, 1, EnumMessageType.RESPONSE),
             '**system**:\nprompt111\n\n**user**:\nsmthU111\n\n**assistant**:\nsmthA222'),
            # Remembering the history (by msg_id), but with a limit
            (('prompt111', 3, True), ('smthU111', 0, 0, EnumMessageType.USER_INPUT),
             '**system**:\nprompt111\n\n**user**:\nsmthU111'),
            (('promptXYZ', None, False), ('smthA222', 0, 1, EnumMessageType.RESPONSE),
             '**system**:\nprompt111\n\n**user**:\nsmthU111\n\n**assistant**:\nsmthA222'),
            ((None, None, False), ('smthU333', 2, 0, EnumMessageType.USER_INPUT),
             '**system**:\nprompt111\n\n**user**:\nsmthU111\n\n**assistant**:\nsmthA222\n\n**user**:\nsmthU333'),
            ((None, None, False), ('smthA444', 2, 4, EnumMessageType.RESPONSE),
             '**system**:\nprompt111\n\n**user**:\nsmthU111\n\n**assistant**:\nsmthA222\n\n**user**:\nsmthU333'
             '\n\n**assistant**:\nsmthA444'),
            ((None, None, False), ('smthA123', 0, 3, EnumMessageType.RESPONSE),
             '**system**:\nprompt111\n\n**user**:\nsmthU111\n\n**assistant**:\nsmthA123\n\n**user**:\nsmthU333'
             '\n\n**assistant**:\nsmthA444'),
        ],
        indirect=True
    )
    def test_add_message(self, mocker, test_prompt_manager: PromptManager,
                         test_params_fixture, test_exp_params_fixture):
        message_text, request_msg_id, response_msg_id, message_type = test_params_fixture
        exp_result = test_exp_params_fixture

        # Retrieve callback methods for verification.
        mock_add_request = mocker.patch.object(test_prompt_manager, 'add_request', wraps=test_prompt_manager.add_request)
        mock_add_response = mocker.patch.object(test_prompt_manager, 'add_response', wraps=test_prompt_manager.add_response)

        test_prompt_manager.add_message(message_text, request_msg_id, response_msg_id, message_type)

        if message_type == EnumMessageType.USER_INPUT:
            mock_add_request.assert_called_once()
            # Check the parameters passed to the mocked function(s)
            assert str(mock_add_request.call_args) == "call('%s', %d)" % (message_text, request_msg_id)
        elif message_type == EnumMessageType.RESPONSE:
            mock_add_response.assert_called_once()
            # Check the parameters passed to the mocked function(s)
            assert (str(mock_add_response.call_args) == "call('%s', %d, %d)"
                    % (message_text, response_msg_id, request_msg_id))
        else:
            mock_add_request.assert_not_called()
            mock_add_response.assert_not_called()

        result = test_prompt_manager.get_history()

        assert result == exp_result

    @pytest.mark.parametrize(
        "test_prompt_manager, test_params_fixture, test_exp_params_fixture",
        [
            # Reload history and append new system prompt
            ((None, None, True), ('', None), [{'system': {'text': '', 'msg_id': None}}]),
            ((None, None, None), ('smth1', None), [{'system': {'text': '', 'msg_id': None}}]),
        ],
        indirect=True
    )
    def test_add_system(self, mocker, test_prompt_manager: PromptManager,
                        test_params_fixture, test_exp_params_fixture):
        system_instruct, msg_id = test_params_fixture
        exp_history_result = test_exp_params_fixture

        test_prompt_manager.add_system(system_instruct, msg_id)

        assert getattr(test_prompt_manager, 'history') == exp_history_result

    @pytest.mark.parametrize(
        "test_prompt_manager, test_params_fixture, test_exp_params_fixture",
        [
            # Reload history and append new request
            ((None, None, True), ('', 0),
             [{'user': {'text': '', 'msg_id': 0}, 'assistant': None}]),
            # Append to the history
            ((None, None, None), ('smth1', 1),
             [{'user': {'text': '', 'msg_id': 0}, 'assistant': None},
              {'user': {'text': 'smth1', 'msg_id': 1}, 'assistant': None}]),
        ],
        indirect=True
    )
    def test_add_request(self, test_prompt_manager: PromptManager,
                         test_params_fixture, test_exp_params_fixture):
        request_text, request_msg_id = test_params_fixture
        exp_history_result = test_exp_params_fixture

        test_prompt_manager.add_request(request_text, request_msg_id)

        assert getattr(test_prompt_manager, 'history') == exp_history_result

    @pytest.mark.parametrize(
        "test_prompt_manager, test_params_fixture, test_exp_params_fixture",
        [
            # Not added
            ((None, None, True), ('', 0, None, [{'user': {'text': '', 'msg_id': 0}}]),
             [{'user': {'text': '', 'msg_id': 0}}]),
            # Added by matching user request msg_id
            ((None, None, True), ('', 0, 0, [{'user': {'text': '', 'msg_id': 0}}]),
             [{'user': {'text': '', 'msg_id': 0}, 'assistant': {'text': '', 'msg_id': 0}}]),
            # Added by a fallback 'assistant': None match
            ((None, None, True), ('', 0, None, [{'user': {'text': '', 'msg_id': 0}, 'assistant': None}]),
             [{'user': {'text': '', 'msg_id': 0}, 'assistant': {'text': '', 'msg_id': 0}}]),
        ],
        indirect=True
    )
    def test_add_response(self, test_prompt_manager: PromptManager,
                          test_params_fixture, test_exp_params_fixture):
        response_text, response_msg_id, request_msg_id, history = test_params_fixture
        exp_history_result = test_exp_params_fixture

        setattr(test_prompt_manager, 'history', history)

        test_prompt_manager.add_response(response_text, response_msg_id, request_msg_id)

        assert getattr(test_prompt_manager, 'history') == exp_history_result

    @pytest.mark.parametrize(
        "test_prompt_manager, test_params_fixture, test_exp_params_fixture",
        [
            # Mocking the history after reload
            ((None, None, True), ([{'system': {}}], ''), None),
            ((None, None, None), ([{'system': {}}], 'system'), {}),
            ((None, None, None), ([
                {'system': {'text': 'system1', 'msg_id': 1}},
                {'user': {'text': 'user1', 'msg_id': 2}},
                {'assistant': {'text': 'assistant1', 'msg_id': 3}}], 'system'),
             {'msg_id': 1, 'text': 'system1'}),
            ((None, None, None), ([
                {'system': {'text': 'system1', 'msg_id': 1}},
                {'user': {'text': 'user1', 'msg_id': 2}},
                {'assistant': {'text': 'assistant1', 'msg_id': 3}}], 'user'),
             {'msg_id': 2, 'text': 'user1'}),
            ((None, None, None), ([
                {'system': {'text': 'system1', 'msg_id': 1}},
                {'user': {'text': 'user1', 'msg_id': 2}},
                {'assistant': {'text': 'assistant1', 'msg_id': 3}}], 'assistant'),
             {'msg_id': 3, 'text': 'assistant1'}),
            ((None, None, None), ([
                {'system': {'text': 'system1', 'msg_id': 1}},
                {'user': {'text': 'user1', 'msg_id': 2}},
                {'assistant': {'text': 'assistant1', 'msg_id': 3}},
                {'user': {'text': 'user2', 'msg_id': 4}},], 'user'),
             {'msg_id': 4, 'text': 'user2'}),
            ((None, None, None), ([
                {'system': {'text': 'system1', 'msg_id': 1}},
                {'user': {'text': 'user2', 'msg_id': 4}},
                {'assistant': {'text': 'assistant1', 'msg_id': 3}},
                {'user': {'text': 'user1', 'msg_id': 2}}, ], 'user'),
             {'msg_id': 2, 'text': 'user1'}),
        ],
        indirect=True
    )
    def test_find_last_message_by_role(self, test_prompt_manager: PromptManager,
                                       test_params_fixture, test_exp_params_fixture):
        history, role = test_params_fixture
        exp_result = test_exp_params_fixture

        setattr(test_prompt_manager, 'history', history)

        result = test_prompt_manager.find_last_message_by_role(role)

        assert result == exp_result



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Part of the 'On-Device LLM' module.
- Functionality: Enables local inference for supported LLMs.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import Qt, Signal
from PySide6.QtWidgets import QVBoxLayout, QWidget, QLabel, QSpinBox, QSlider, QSizePolicy, QScrollArea

import os
import logging

import asyncio
from asyncio import Task
from qasync import asyncSlot

from typing import TYPE_CHECKING, Callable

from .model_helper import ModelHelper
from .prompt_manager import PromptManager

from ..base_ai_core import BaseAiCore

from .. import Settings
from .. import AppConfig
from .. import Lexemes

from ...ui.ai_assistant.ai_assistant import EnumMessageType, EnumMessageStyle
from ...ui.dir_path_line_edit import DirPathLineEdit
from ...ui.horizontal_line_spacer import HorizontalLineSpacer
from ...ui.label_with_hint import LabelWithHint

if TYPE_CHECKING:
    # ONNX Runtime GenAI
    from onnxruntime_genai import Generator  # noqa


class ModuleCore(BaseAiCore):
    update_signal = Signal(str, object, object, EnumMessageType, EnumMessageStyle)
    # prompt_tokens, completion_tokens, total_tokens
    update_usage_signal = Signal(str, object, object, object, object)

    # Default name, use lexeme for the translated one
    module_name = 'On Device LLM'

    # Functionality extended by the module.
    extensions = ['ai_assistant', 'settings_dialog']

    init_callback: Callable = None
    finished_callback: Callable = None

    generator_task: Task = None

    prompt_manager: PromptManager = None

    def __init__(self):
        super(ModuleCore, self).__init__()

        self.settings = Settings(parent=self)
        self.settings.value_changed.connect(
            lambda v: self.settings_update_handler(v))

        self.logger = logging.getLogger('module_ondevice_llm')

        # Load lexemes for selected language and scope
        self.lexemes = Lexemes(self.settings.app_language,
                               default_scope='settings_dialog',
                               lexemes_dir=self.get_lexemes_path())

        # Extend Settings explicitly as it's a global object which may cause circular recursion
        self.extend_settings_create_property(self.settings.create_property)
        # Add this module to the inference modules selection list
        inference_modules = self.settings.ai_config_inference_modules
        default_module = [key for key, value in inference_modules.items() if isinstance(value, tuple) and True in value]
        inference_modules.update({'ondevice_llm': (self.lexemes.get('module_ondevice_llm_name', scope='common'),
                                                   True if not default_module else False)})
        self.settings.ai_config_inference_modules = inference_modules

        model_path = self.settings.module_ondevice_llm_model_path
        search_options = self.get_search_options()
        # Cached helper instance
        self.model_helper = ModelHelper(model_path=model_path, search_options=search_options)

        # Just in case of debug of async events
        # asyncio.get_event_loop().set_debug(True)

        self.logger.debug(f'Module {__name__} loaded')

    def get_search_options(self):
        search_options = {}
        if hasattr(self.settings, 'module_ondevice_llm_response_temperature'):
            search_options.update({'temperature': self.settings.module_ondevice_llm_response_temperature})
        if (hasattr(self.settings, 'module_ondevice_llm_response_max_tokens')
                and self.settings.module_ondevice_llm_response_max_tokens > 0):
            search_options.update({'max_length': self.settings.module_ondevice_llm_response_max_tokens})
        return search_options

    def init_prompt_manager(self, ai_dialog):
        """
        Prompt manager for prompt management and to add / append messages to prompt history.
        Note: It may not be initiated if called out of the context, from settings for example.
        """
        self.prompt_manager = PromptManager(max_history_size=self.settings.module_ondevice_llm_prompt_history_size,
                                            parent=ai_dialog)

    def get_prompt_manager(self):
        return self.prompt_manager

    @staticmethod
    def get_lexemes_path():
        return os.path.join(os.path.dirname(__file__), 'lexemes')

    def attach_dialog(self, ai_dialog):
        # Catch parent dialog closing event
        ai_dialog.dialog_closed.connect(self.parent_closed)
        # Request cancelled event
        ai_dialog.request_cancelled.connect(self.request_cancelled)
        # Connect update signal to the dialog
        self.update_signal.connect(ai_dialog.append_to_message)
        # Connect to the token usage update signal
        self.update_usage_signal.connect(ai_dialog.update_usage)
        # Init and connect prompt manager
        self.init_prompt_manager(ai_dialog)

    @asyncSlot()
    async def parent_closed(self):
        ai_dialog = self.sender()
        # try/except because of an object might be deleted during processing callbacks asynchronously.
        # Explicitly check everything is disconnected.
        try:
            if ai_dialog:
                # Disconnect from the messages updating signal
                self.update_signal.disconnect(ai_dialog.append_to_message)
                # Disconnect from the token usage update signal
                self.update_usage_signal.disconnect(ai_dialog.update_usage)
                if hasattr(ai_dialog, 'dialog_closed'):
                    # Parent window might be closed
                    ai_dialog.dialog_closed.disconnect(self.parent_closed)
        except RuntimeError as e:  # Object might be already deleted
            self.logger.warning(f'Error occurred during the closing process {e}')
        # Clear prompt history
        PromptManager.reload()
        # Stop the generator thread before the dialog closes
        await self.stop_generator()

    @asyncSlot()
    async def request_cancelled(self):
        # Stop the generator thread before the dialog closes
        await self.stop_generator()

    @asyncSlot()
    async def request(self, user_prompt: str, request_msg_id: int, response_msg_id: int,
                      init_callback: Callable = None, finished_callback: Callable = None):

        if not asyncio.get_running_loop().is_running():
            return

        # Check the prompt manager is activated
        if not hasattr(self, 'prompt_manager') or self.prompt_manager is None:
            return

        # Store callbacks
        self.init_callback = init_callback
        self.finished_callback = finished_callback

        # Init model
        try:
            self.model_helper.init_model()
        except (AttributeError, Exception) as e:
            self.logger.error(f'{e}')
            # Complete init
            if self.init_callback and callable(self.init_callback):
                self.init_callback()
            # Error message to show in UI
            outputs = self.lexemes.get("module_ondevice_llm_model_exception", scope='common', error_msg=str(e))
            # Emit update message signal
            self.update_signal.emit(outputs, None, None, EnumMessageType.DEFAULT, EnumMessageStyle.ERROR)
            # Emit finished signal
            self.finished_callback(request_msg_id=request_msg_id, response_msg_id=response_msg_id,
                                   message_type=EnumMessageType.RESPONSE)
            return

        input_tokens = self.model_helper.get_input_tokens(user_prompt)

        # Emit update usage signal
        self.update_usage_signal.emit(self.get_model_name(), len(input_tokens), 0, len(input_tokens), False)

        if not self.generator_task or self.generator_task.done():
            # self.generator_task = await asyncio.create_task(self.run_generator(search_options, input_tokens))
            self.generator_task = asyncio.ensure_future(
                self.run_generator(input_tokens, request_msg_id, response_msg_id))
            # Finished callback
            if self.finished_callback and callable(self.finished_callback):
                self.generator_task.add_done_callback(
                    lambda _task, _request_msg_id=request_msg_id, _response_msg_id=response_msg_id:
                    self.finished_callback(request_msg_id=_request_msg_id, response_msg_id=_response_msg_id,
                                           message_type=EnumMessageType.RESPONSE))
            # Run task asynchronously
            done, pending = await asyncio.wait([self.generator_task], return_when=asyncio.FIRST_COMPLETED)

            for task in done:
                if not task.cancelled():
                    exception = task.exception()
                    if exception:
                        self.logger.warning(f"Task raised an exception: {exception}")
                        # Error message to show in UI
                        outputs = self.lexemes.get('module_ondevice_llm_task_exception', scope='common',
                                                   error_msg=str(exception))
                        # Emit update message signal
                        self.update_signal.emit(outputs, None, None, EnumMessageType.DEFAULT, EnumMessageStyle.ERROR)
                    else:
                        result = task.result()
                        self.logger.debug(f"Task completed with result: {result}")

    async def run_generator(self, input_tokens, request_msg_id, response_msg_id):

        try:
            generator = self.model_helper.init_generator(input_tokens, ModelHelper.search_options)  # type: Generator
        except ModuleNotFoundError as e:
            self.logger.error(f'Cannot init model generator: {e}', exc_info=False)
            # Re-raise to indicate unresolved issues to the caller
            raise
        finally:
            # The init stage has concluded
            if self.init_callback and callable(self.init_callback):
                self.init_callback()

        try:
            while not generator.is_done():
                await self.async_generator(request_msg_id, response_msg_id)
                await asyncio.sleep(0.05)  # Sleep to allow UI to update
        except asyncio.CancelledError:
            self.logger.info("Generation cancelled")
        except (WindowsError, RecursionError) as e:
            self.logger.error(f"Error occurred: {e}")
            # raise
        except (SystemExit, Exception) as e:
            self.logger.error(f"Exception raised: {e}")
            # raise
        finally:  # Generation ended
            await self.stop_generator()

    async def async_generator(self, request_msg_id, response_msg_id):
        # Get outputs for this iteration
        outputs = self.model_helper.generate_output()
        if outputs:
            # Emit update message signal
            self.update_signal.emit(outputs, request_msg_id, response_msg_id,
                                    EnumMessageType.RESPONSE, EnumMessageStyle.DEFAULT)
            # Emit update usage signal
            self.update_usage_signal.emit(self.get_model_name(), 0, 1, 1, True)  # One token at the moment

    async def stop_generator(self):
        # Cancel async task(s)
        if self.generator_task and not self.generator_task.done():
            # Allow to finish callback, do not remove:
            # > self.generator_task.remove_done_callback(self.finished_callback)
            self.generator_task.cancel()

    def extend_settings_dialog_fields_conf(self, tab_widget) -> list:
        # On Device LLM Config
        tab_ondevice_llm_config = QWidget()

        # Create the scroll area
        scroll_area = QScrollArea()
        scroll_area.setObjectName('settings_dialog_tab_ondevice_llm_config')
        scroll_area.setWidgetResizable(True)

        # Define the layout for the On Device LLM configuration tab
        tab_ondevice_llm_config_layout = QVBoxLayout(tab_ondevice_llm_config)

        # Set the content widget inside the scroll area
        scroll_area.setWidget(tab_ondevice_llm_config)

        tab_widget.addTab(scroll_area, self.lexemes.get('tab_ondevice_llm_config'))

        return [
            # [On Device LLM config]
            # Label for the configuration block
            {"type": QLabel, "name": "settings_dialog_module_ondevice_llm_config_label",
             "props": {"setProperty": ("class", "group-header-label")},
             "alignment": Qt.AlignmentFlag.AlignLeft,
             "text": self.lexemes.get('module_ondevice_llm_config_label'), "style": {"bold": True},
             "callback": lambda obj: tab_ondevice_llm_config_layout.addWidget(obj)},
            # Label for the model path input field
            {"type": LabelWithHint, "kwargs": {
                "tooltip": ('module_ondevice_llm_config_path_input_accessible_description',
                            self.lexemes.get('module_ondevice_llm_config_path_input_accessible_description'))},
             "name": "settings_dialog_module_ondevice_llm_config_path_label",
             "alignment": Qt.AlignmentFlag.AlignLeft,
             "text": self.lexemes.get('module_ondevice_llm_config_path_label'),
             "callback": lambda obj: tab_ondevice_llm_config_layout.addWidget(obj, alignment=Qt.AlignmentFlag.AlignTop)},
            # Input field for the model path
            {"type": DirPathLineEdit, "kwargs": {"settings": self.settings},
             "name": "settings_dialog_ondevice_llm_config_path:module_ondevice_llm_model_path",
             "read_only": False, "max_length": 256,
             "callback": lambda obj: tab_ondevice_llm_config_layout.addWidget(obj, alignment=Qt.AlignmentFlag.AlignTop),
             "placeholder_text": self.lexemes.get('module_ondevice_llm_config_path_input_placeholder_text'),
             "accessible_description":
                 self.lexemes.get('module_ondevice_llm_config_path_input_accessible_description')},
            # Vertical spacer
            {"type": QWidget, "name": None, "size_policy": (QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Minimum),
             "callback": lambda obj: tab_ondevice_llm_config_layout.addWidget(obj)},
            # Label for the temperature slider
            {"type": LabelWithHint, "kwargs": {
                "tooltip": ('module_ondevice_llm_config_response_temperature_input_accessible_description',
                            self.lexemes.get('module_ondevice_llm_config_response_temperature_input_accessible_description'))},
             "name": "settings_dialog_module_ondevice_llm_config_response_temperature_label",
             "alignment": Qt.AlignmentFlag.AlignLeft,
             "text": self.lexemes.get('module_ondevice_llm_config_response_temperature_label',
                                      temperature=self.settings.module_ondevice_llm_response_temperature),
             "callback": lambda obj: tab_ondevice_llm_config_layout.addWidget(obj, alignment=Qt.AlignmentFlag.AlignTop)},
            # Slider to adjust the temperature setting
            {"type": QSlider, "args": [Qt.Orientation.Horizontal],
             "props": {'setFocusPolicy': Qt.FocusPolicy.StrongFocus, 'setTickPosition': QSlider.TickPosition.TicksAbove,
                       'setTickInterval': 5, 'setSingleStep': 5, 'setMinimum': 0, 'setMaximum': 100},
             "name": "settings_dialog_module_ondevice_llm_config_response_temperature:"
                     "module_ondevice_llm_response_temperature",  # Lexeme key : Object name
             "callback": lambda obj: tab_ondevice_llm_config_layout.addWidget(obj),
             "on_value_change": self.temperature_change_handler,
             "accessible_description":
                 self.lexemes.get('module_ondevice_llm_config_response_temperature_input_accessible_description')},
            # Vertical spacer
            {"type": QWidget, "name": None, "size_policy": (QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Minimum),
             "callback": lambda obj: tab_ondevice_llm_config_layout.addWidget(obj)},
            # Label for the maximum token count setting
            {"type": LabelWithHint, "kwargs": {
                "tooltip": ('module_ondevice_llm_config_response_max_tokens_input_accessible_description',
                            self.lexemes.get('module_ondevice_llm_config_response_max_tokens_input_accessible_description'))},
             "name": "settings_dialog_module_ondevice_llm_config_response_max_tokens_label",
             "alignment": Qt.AlignmentFlag.AlignLeft,
             "text": self.lexemes.get('module_ondevice_llm_config_response_max_tokens_label'),
             "callback": lambda obj: tab_ondevice_llm_config_layout.addWidget(obj, alignment=Qt.AlignmentFlag.AlignTop)},
            # Input field for the maximum token count setting
            {"type": QSpinBox, "props": {'setMinimum': 0, 'setMaximum': 65536},  # Update the highest range
             "size_policy": (QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Preferred),
             "name": "settings_dialog_module_ondevice_llm_config_response_max_tokens:"
                     "module_ondevice_llm_response_max_tokens",
             "callback": lambda obj: tab_ondevice_llm_config_layout.addWidget(obj, alignment=Qt.AlignmentFlag.AlignTop),
             "accessible_description":
                 self.lexemes.get('module_ondevice_llm_config_response_max_tokens_input_accessible_description')},
            # Horizontal line spacer
            {"type": HorizontalLineSpacer, "callback": lambda obj: tab_ondevice_llm_config_layout.addWidget(obj)},
            # Label for the prompt history maximum capacity settings
            {"type": LabelWithHint, "kwargs": {
                "tooltip": ('module_ondevice_llm_config_prompt_history_size_input_accessible_description',
                            self.lexemes.get('module_ondevice_llm_config_prompt_history_size_input_accessible_description'))},
             "name": "settings_dialog_module_ondevice_llm_config_prompt_history_size_label",
             "alignment": Qt.AlignmentFlag.AlignLeft,
             "text": self.lexemes.get('module_ondevice_llm_config_prompt_history_size_label'),
             "callback": lambda obj: tab_ondevice_llm_config_layout.addWidget(obj, alignment=Qt.AlignmentFlag.AlignTop)},
            # Input field for the prompt history maximum capacity setting
            {"type": QSpinBox, "props": {'setMinimum': 0, 'setMaximum': 65536},  # Update the highest range
             "size_policy": (QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Preferred),
             "name": "settings_dialog_module_ondevice_llm_config_prompt_history_size:"
                     "module_ondevice_llm_prompt_history_size",
             "callback": lambda obj: tab_ondevice_llm_config_layout.addWidget(obj, alignment=Qt.AlignmentFlag.AlignTop),
             "accessible_description":
                 self.lexemes.get('module_ondevice_llm_config_prompt_history_size_input_accessible_description')},
            # Spacer to align elements at the top of the layout
            {"type": QWidget, "name": None, "size_policy": (QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding),
             "callback": lambda obj: tab_ondevice_llm_config_layout.addWidget(obj)},
        ]

    def get_model_name(self):
        # Get LLM model name
        return f'({self.model_helper.get_model_name()})'

    @staticmethod
    def extend_settings_create_property(extend_func: Callable):
        if callable(extend_func):
            extend_func("module_ondevice_llm_model_path", str, "")
            extend_func("module_ondevice_llm_response_temperature", int, 20)
            extend_func("module_ondevice_llm_response_max_tokens", int, 0)
            extend_func("module_ondevice_llm_prompt_history_size", int, 0)

    def settings_update_handler(self, data) -> None:
        """
        Perform actions upon settings change.
        Data comes in a view of a dictionary, where is the key is the setting name, and the value is the actual value.
        Can be resource greedy.

        Args:
            data dict: say {"module_ondevice_llm_model_path": "..."}

        Return:
            None
        """

        AppConfig().logger.debug('Settings update handler is in use "%s"' % data)

        options = [
            'module_ondevice_llm_model_path',
            'module_ondevice_llm_response_temperature',
            'module_ondevice_llm_response_max_tokens',
        ]
        if any(option in data for option in options) or 'ai_config_inference_module' in data:
            # Set up updated value or one from settings
            model_path = self.settings.module_ondevice_llm_model_path
            if 'module_ondevice_llm_model_path' in data:
                model_path = data['module_ondevice_llm_model_path']

            if model_path:
                # Reload model helper
                ModelHelper.reload()

                search_options = self.get_search_options()

                # The model will be reloaded next time with
                self.model_helper = ModelHelper(model_path=model_path, search_options=search_options)

    def temperature_change_handler(self, source_object, source_widget):
        if source_object.objectName() and source_widget:
            # Parse the object name in case it contains a combination of lexeme and setting keys
            _lexeme_key, setting_name = self.settings.settings_helper.parse_object_name(source_object.objectName())
            if setting_name == 'module_ondevice_llm_response_temperature':
                # Get the latest value from the object, as settings might not yet be updated
                temperature_int = source_object.value()  # source_object type: QSlider
                temperature_float = self.model_helper.convert_temperature(temperature_int)
                temperature_label = source_widget.findChild(
                    QLabel,
                    'settings_dialog_module_ondevice_llm_config_response_temperature_label'
                )
                if temperature_label:
                    temperature_label.setText(
                        self.lexemes.get('module_ondevice_llm_config_response_temperature_label',
                                         temperature=temperature_float))



# German lexemes main_menu.py
lexemes = {
    "group_file_label": "Datei",
    "group_edit_label": "Bearbeiten",
    "group_tools_label": "Werkzeuge",
    "group_help_label": "Hilfe",

    "actions_file_label_new_document": "Neues Dokument",
    "actions_file_accessible_name_new_document": "Neues Dokument",
    "actions_file_label_open": "Öffnen",
    "actions_file_accessible_name_open": "Dokument öffnen",
    "actions_file_label_save": "Speichern",
    "actions_file_accessible_name_save": "Dokument speichern",
    "actions_file_label_save_as": "Speichern unter",
    "actions_file_accessible_name_save_as": "Kopie des Dokuments speichern",
    "actions_file_label_settings": "Einstellungen",
    "actions_file_accessible_name_settings": "App-Einstellungen",
    "actions_file_label_reset_settings": "Einstellungen zurücksetzen",
    "actions_file_accessible_name_reset_settings": "Einstellungen zurücksetzen",
    "actions_file_label_exit": "Beenden",
    "actions_file_accessible_name_exit": "App beenden",

    "actions_edit_edit_mode": "Bearbeiten",
    "actions_edit_accessible_name_edit_mode": "Datei bearbeiten",
    "actions_edit_source_mode": "Quelle",
    "actions_edit_accessible_name_source_mode": "Quellcode",
    "actions_edit_bold": "Fett",
    "actions_edit_accessible_name_bold": "Fettschrift Format",
    "actions_edit_italic": "Kursiv",
    "actions_edit_accessible_name_italic": "Kursivschrift Format",
    "actions_edit_underline": "Unterstreichen",
    "actions_edit_accessible_name_underline": "Unterstrichen Textformat",
    "actions_edit_strikethrough": "Durchgestrichen",
    "actions_edit_accessible_name_strikethrough": "Durchgestrichenes Textformat",
    "actions_edit_blockquote": "Blockzitat",
    "actions_edit_accessible_name_blockquote": "Blockzitat Textformat",

    "actions_tools_ai_assistant": "KI-Assistent",
    "actions_tools_accessible_name_ai_assistant": "KI-Assistent starten",
    "actions_tools_color_picker": "Farbwähler",
    "actions_tools_accessible_name_color_picker": "Textfarbe auswählen",

    "actions_help_label_md_syntax": "Markdown-Syntax",
    "actions_help_accessible_name_md_syntax": "Markdown-Syntax",
    "actions_help_label_check_for_updates": "Auf Updates prüfen",
    "actions_help_accessible_name_check_for_updates": "Auf Updates prüfen",
    "actions_help_label_bug_report": "Fehlerbericht",
    "actions_help_accessible_name_bug_report": "Fehlerbericht senden",
    "actions_help_label_about": "Über",
    "actions_help_accessible_name_about": "Über",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Part of the base module.
- Functionality: Base class for modules that extend AI Assistant functionality.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from abc import abstractmethod

from .base_core import BaseCore


class BaseAiCore(BaseCore):

    @abstractmethod
    def get_prompt_manager(self):
        # Subclasses are expected to override this method.
        raise NotImplementedError("Subclasses must implement get_prompt_manager()")

    @abstractmethod
    def request(self, *args, **kwargs):
        # Subclasses are expected to override this method.
        raise NotImplementedError("Subclasses must implement request()")



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Password object class to store password-related data.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""


class EncPassword:

    def __init__(self):
        self._password = None
        self._hint = None

    @property
    def password(self):
        return self._password

    @password.setter
    def password(self, value):
        self._password = value

    @property
    def hint(self):
        return self._hint

    @hint.setter
    def hint(self, value):
        self._hint = value

    def is_valid(self):
        if self._password is None:
            return False
        if len(self._password) < 8:
            return False
        return True



# Swedish lexemes ai_assistant.py
lexemes = {
    # AI assistant dialog
    "dialog_title": "Förfrågan till AI",
    "dialog_prompt_input_placeholder_text": "Förfrågan",
    "dialog_prompt_input_accessible_description": "Ange en förfrågan för att ge kontext till AI-assistenten",
    "dialog_response_output_placeholder_text": "Svarsdata",
    "dialog_response_output_accessible_description": "Svarsdata kommer att visas i detta fält",
    "dialog_response_output_notice_empty_text": "Ange text för att få ett svar om...",

    "dialog_usage_module_label": "MODUL",
    "dialog_usage_model_label": "MODELL",
    "dialog_usage_tokens_label": "TOKENS",
    "dialog_usage_tokens_prompt": "prompt: {tokens}",
    "dialog_usage_tokens_answer": "svar: {tokens}",
    "dialog_usage_tokens_total": "totalt: {tokens}",

    "dialog_button_send_request": "Skicka förfrågan",
    "dialog_button_stop_request": "Stoppa",
    "dialog_button_save_history": "Spara dialoghistorik",

    "dialog_message_copy_tooltip": "Kopiera text",
    "dialog_message_copied_tooltip": "Kopierad",

    "dialog_error_loading_model": "Fel vid laddning av anpassad modell",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Part of the 'OpenAI API' module.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

# This import is essential for proper module loading.
from .module_core import ModuleCore


def get_name():
    """
    Readable module name.
    """
    return ModuleCore.module_name


def is_available() -> bool:
    """
    No additional requirements.
    """
    return True



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Contains unit and integration tests for the related functionality.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

import pytest

from .. import is_module_available


def pytest_collection_modifyitems(config, items):
    """
    Conditionally skip tests in a specific directory.
    """
    # Check if the condition is not met
    if not is_module_available('llama_cpp'):
        skip_marker = pytest.mark.skip(reason="Condition is not met. Skipping tests in 'llama_cpp'.")
        # Skip all tests in the 'llama_cpp' directory
        for item in items:
            if 'llama_cpp' in item.nodeid:
                item.add_marker(skip_marker)



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Enum class for the app's supported fonts.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

import os

from .enum_base import EnumBase

assert hasattr(EnumBase, 'default'), "Check default() method is implemented in the base class"


class Fonts(EnumBase):
    NOTO_SANS = (os.path.join('NotoSans', 'NotoSans-Regular.ttf'), 'Noto Sans', False, True)
    IBM_PLEX_MONO = (os.path.join('IBMPlexMono', 'IBMPlexMono-Regular.ttf'), 'IBM Plex Mono', True)

    def __init__(self, path, display_name, is_monospace, is_default=False, *args, **kwargs):
        super().__init__(path, is_default=is_default, *args, **kwargs)

        self._display_name = display_name
        self._is_monospace = is_monospace

    @property
    def path(self):
        # Get the font file path.
        return self.value

    @property
    def display_name(self):
        # Get the display name of the font.
        return self._display_name

    @property
    def is_monospace(self):
        # Check if the font is monospace.
        return self._is_monospace

    @staticmethod
    def get_by_name(name):
        # Retrieve a font by its display name.
        for font in Fonts:
            if font.display_name == name:
                return font
        return None

    @staticmethod
    def get_all_monospace():
        # Retrieve all monospace fonts.
        return [font for font in Fonts if font.is_monospace]



# Portuguese lexemes settings_dialog.py
lexemes = {
    "tab_module_llama_cpp_config": "Módulo llama.cpp",

    "module_llama_cpp_config_label": "Módulo llama.cpp",
    "module_llama_cpp_config_path_label": "Localização do Modelo",
    "module_llama_cpp_config_path_input_placeholder_text": "Selecione ou digite o caminho do modelo",
    "module_llama_cpp_config_path_input_accessible_description":
        "Um campo de entrada com um seletor para especificar o caminho do modelo local. Suporta modelos no formato GGUF,\n"
        "um formato de arquivo binário otimizado para armazenar modelos usados com GGML e executores baseados em GGML.",
    "module_llama_cpp_config_path_input_filter_text": "Arquivos GGUF",

    "module_llama_cpp_config_context_window_label": "Tamanho da Janela de Contexto",
    "module_llama_cpp_config_context_window_input_accessible_description":
        "Define o número de tokens que o modelo considera para gerar respostas. Controla quanto do contexto anterior é usado.",

    "module_llama_cpp_chat_formats_label": "Formatos de Chat",
    "module_llama_cpp_chat_formats_combo_placeholder_text": "Selecione um formato de chat",
    "module_llama_cpp_chat_formats_combo_accessible_description":
        "Menu suspenso para selecionar o formato usado para conversas do modelo.",

    "module_llama_cpp_config_system_prompt_label": "Prompt do Sistema",
    "module_llama_cpp_config_system_prompt_edit_placeholder_text": "Digite o texto do prompt do sistema",
    "module_llama_cpp_config_system_prompt_edit_accessible_description":
        "Campo de texto para inserir prompts de sistema que orientam as respostas do modelo.",

    "module_llama_cpp_config_response_temperature_label": "Temperatura de Resposta: {temperature}",
    "module_llama_cpp_config_response_temperature_input_accessible_description":
        "Ajusta a aleatoriedade das respostas do modelo. Valores mais altos produzem saídas mais variadas,\n"
        "enquanto valores mais baixos resultam em respostas mais previsíveis.",

    "module_llama_cpp_config_response_max_tokens_label": "Máximo de Tokens por Resposta",
    "module_llama_cpp_config_response_max_tokens_input_accessible_description":
        "Limita o número de tokens nas respostas do modelo até o limite real da janela de contexto.\n"
        "Um valor zero assume a capacidade da janela de contexto.",

    "module_llama_cpp_config_prompt_history_size_label": "Tamanho do Histórico de Prompts",
    "module_llama_cpp_config_prompt_history_size_input_accessible_description":
        "Controla o número de entradas no histórico de prompts mantidas pelo sistema para referência.\n"
        "Um valor zero permite entradas ilimitadas."
}



# English lexemes settings_dialog.py
lexemes = {
    # Settings dialog
    "window_title": "Settings",

    "button_close": "Close",

    "tab_general": "General",
    "tab_editor_config": "Editor",
    "tab_viewer_config": "Viewer",
    "tab_ai_config": "AI Config",

    "general_app_config_label": "App config",
    "general_app_language_label": "Language",
    "general_app_language_combo_placeholder_text": "Choose a language",
    "general_app_language_combo_accessible_description": "App's interface language",
    "general_app_theme_label": "Theme",
    "general_app_theme_combo_placeholder_text": "Choose a theme",
    "general_app_theme_combo_accessible_description": "App's interface theme",
    "general_app_main_menu_label": "Main Menu",
    "general_app_main_menu_checkbox": "Show Main Menu",
    "general_app_main_menu_checkbox_accessible_description": "Display the app's main dropdown menu",
    "general_app_font_size_label": "Font Size: {size}pt",
    "general_app_font_size_slider_accessible_description": "Adjust the app's global font size",

    "general_statusbar_label": "Status Bar",
    "general_statusbar_show_global_cursor_position_checkbox": "Show Global Cursor Position",
    "general_statusbar_show_global_cursor_position_checkbox_accessible_description":
        "Display the global cursor position in the status bar",

    "editor_config_label": "Editor Configuration",
    "editor_config_show_line_numbers_checkbox": "Show Line Numbers",
    "editor_config_show_line_numbers_checkbox_accessible_description": "Display line numbers in the editor",

    "viewer_config_label": "Viewer Configuration",
    "viewer_config_process_emojis_checkbox": "Convert Text Emojis to Graphics",
    "viewer_config_process_emojis_checkbox_accessible_description": "Convert text emojis to graphical representations",
    "viewer_config_highlight_todos_checkbox": "Highlight TODOs",
    "viewer_config_highlight_todos_checkbox_accessible_description": "Emphasize TODO tags within the text",
    "viewer_config_open_link_confirmation_checkbox": "Require Confirmation to Open Links",
    "viewer_config_open_link_confirmation_checkbox_accessible_description":
        "Ask for confirmation before opening links",
    "viewer_config_save_resources_checkbox": "Auto-save external images to disk",
    "viewer_config_save_resources_checkbox_accessible_description":
        "Automatically saves copies of external images to disk for offline access.",

    "ai_config_inference_module_label": "Inference Module",
    "ai_config_inference_module_names_combo_label": "Active Inference Module",
    "ai_config_inference_module_names_combo_placeholder_text": "Choose Module",
    "ai_config_inference_module_names_combo_accessible_description":
        "Select from available AI inference modules to operate with the AI Assistant.\n"
        "Options include local Large Language Models (LLM) with real-time processing, or API-based functionalities.",

    "ai_config_base_label": "Base Parameters",
    "ai_config_multi_turn_dialogue_checkbox": "Multi-turn dialogue with conversational memory",
    "ai_config_multi_turn_dialogue_checkbox_accessible_description":
        "Enable multi-turn dialogue that retains the previous prompt for conversational memory.\n"
        "When switched off, only the new message and the system prompt influence the response.",
    "ai_config_convert_to_md_checkbox": "Convert the result to Markdown",
    "ai_config_convert_to_md_checkbox_accessible_description": "Convert the output message into Markdown format.",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: About the app dialog class for displaying app's info to the user.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import Qt, QEvent, QUrl
from PySide6.QtGui import QPixmap, QCursor, QDesktopServices
from PySide6.QtWidgets import QHBoxLayout, QVBoxLayout, QGridLayout, QDialog, QLabel, QFrame, QPushButton, QWidget
from PySide6.QtWidgets import QSizePolicy

from . import Settings
from . import AppConfig
from . import Lexemes
from . import ThemeHelper

from functools import partial

import logging


class AboutPopup(QDialog):
    def __init__(self, parent=None):
        # Popup type may block a screen lock action.
        # Popup is not closing on macOS and no pointing hand cursor.
        # Qt.WindowType.Dialog | Qt.WindowType.FramelessWindowHint
        super().__init__(parent, Qt.WindowType.Dialog)

        self.parent = parent

        # Apply font from the dialog instance to the label
        self.setFont(self.parent.font())

        self.settings = Settings(parent=self)

        self.logger = logging.getLogger('about_popup')

        # Load lexemes for selected language and scope
        self.lexemes = Lexemes(self.settings.app_language, default_scope='common')

        # Theme helper
        self.theme_helper = ThemeHelper()

        self.init_ui()

        self.setModal(True)  # Set the dialog modal to manage focus more effectively

        # self.adjustSize()  # Adjust size based on content
        """
        main_window_size = self.parent.size()
        dialog_width = int(main_window_size.width() * 0.25)
        dialog_height = int(main_window_size.height() * 0.25)
        # Set dialog size derived from the main window size
        self.setMinimumSize(dialog_width, dialog_height)
        """

        self.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)
        if self.sizeHint().isValid():
            self.setMinimumSize(self.sizeHint())

    def init_ui(self):
        self.setWindowTitle(self.lexemes.get('popup_about_title'))

        layout = QVBoxLayout(self)
        self.setLayout(layout)

        frame = QFrame(self)
        frame_layout = QGridLayout(frame)
        frame_layout.setVerticalSpacing(15)
        frame_layout.setHorizontalSpacing(25)
        frame.setLayout(frame_layout)

        # Header with an icon and app name
        icon_label = QLabel(self)
        icon_label.setObjectName('app_icon')
        pixmap = (QPixmap(self.theme_helper.get_app_icon_path())
                  .scaled(48, 48, Qt.AspectRatioMode.KeepAspectRatio,
                          Qt.TransformationMode.SmoothTransformation))
        icon_label.setPixmap(pixmap)
        frame_layout.addWidget(icon_label, 0, 1, Qt.AlignmentFlag.AlignLeft, 1)

        app_name_widget = QWidget(self)
        app_name_layout = QVBoxLayout(app_name_widget)
        app_name_layout.setSpacing(0)
        app_name_layout.setContentsMargins(0, 0, 0, 0)

        app_name = QLabel(AppConfig().get_app_name(), app_name_widget)
        app_name_font = self.font()
        app_name_font.setPointSizeF(app_name_font.pointSize() * 1.7)
        app_name.setFont(app_name_font)
        app_name.setObjectName('app_name')
        app_name_layout.addWidget(app_name)
        app_name_caption = QLabel(self.lexemes.get('popup_about_app_name_description'), app_name_widget)
        app_name_caption.setObjectName('app_name_caption')
        app_name_layout.addWidget(app_name_caption)

        frame_layout.addWidget(app_name_widget, 0, 2, Qt.AlignmentFlag.AlignLeft, 2)

        # Information fields
        info_fields = [
            (self.lexemes.get('popup_about_version'), AppConfig().get_app_version(), []),
            (self.lexemes.get('popup_about_license'), AppConfig().get_app_license(), []),
            (self.lexemes.get('popup_about_website'), AppConfig().get_app_website(),
             [{'icon': 'box-arrow-up-right.svg', 'link': AppConfig().get_app_website()}]),
            (self.lexemes.get('popup_about_repository'), AppConfig().get_app_repository(),
             [{'icon': 'star-fill.svg', 'link': AppConfig().get_app_repository()}]),
            (self.lexemes.get('popup_about_pypi'), AppConfig().get_app_pypi(),
             [{'icon': 'box-arrow-up-right.svg', 'link': AppConfig().get_app_pypi()}]),
            (self.lexemes.get('popup_about_date'), AppConfig().get_app_date(), [])
        ]

        for i, row in enumerate(info_fields):
            label, value, value_icons = row
            # Label widget
            label_widget = QLabel(f"{label}:", self)
            label_widget.setObjectName('label_widget')
            # Add label widget to the layout
            frame_layout.addWidget(label_widget, i+1, 1, Qt.AlignmentFlag.AlignLeft, 1)

            # Value widget
            value_widget = QWidget(self)
            value_widget.setObjectName('value_widget')
            value_widget.setSizePolicy(QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Minimum)
            value_layout = QHBoxLayout()
            value_layout.setContentsMargins(0, 0, 0, 0)
            value_layout.setSpacing(5)
            value_layout.setAlignment(Qt.AlignmentFlag.AlignLeft)
            value_widget.setLayout(value_layout)

            if value.startswith('http'):
                value_widget_link = QPushButton(value, self)
                value_widget_link.setSizePolicy(QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Minimum)
                value_widget_link.setObjectName('value_widget_link')
                value_widget_link.clicked.connect(partial(lambda v: QDesktopServices.openUrl(v), value))
                value_widget_link.setCursor(QCursor(Qt.CursorShape.PointingHandCursor))
                """
                # In case of adding icon ahead of the link:
                value_widget_icon = self.theme_helper.get_icon(theme_icon='...')
                value_widget.setIcon(value_widget_icon)
                """
                value_layout.addWidget(value_widget_link, alignment=Qt.AlignmentFlag.AlignLeft)
            else:
                value_widget_text = QLabel(value, self)
                value_widget_text.setSizePolicy(QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Minimum)
                value_layout.addWidget(value_widget_text, alignment=Qt.AlignmentFlag.AlignLeft)

            if value_icons:
                for value_icon in value_icons:
                    value_widget_icon_link = QPushButton(self)
                    value_widget_icon_link.setObjectName('value_widget_icon_link')
                    value_widget_icon_link.setCursor(QCursor(Qt.CursorShape.PointingHandCursor))
                    value_widget_icon_link.setSizePolicy(QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Minimum)
                    value_widget_icon = self.theme_helper.get_icon(
                        theme_icon=value_icon['icon'], color=self.theme_helper.get_color('popup_about_icon_link'))
                    # If using QLabel instead of QPushButton, the pixmap could be set like this:
                    # value_widget_icon_link.setPixmap(value_widget_icon.pixmap(16, 16))
                    value_widget_icon_link.setIcon(value_widget_icon)
                    value_widget_icon_link.clicked.connect(
                        partial(lambda url: QDesktopServices.openUrl(QUrl(url)), value_icon['link']))
                    value_layout.addWidget(value_widget_icon_link, alignment=Qt.AlignmentFlag.AlignLeft)

            # Add value widget to the layout
            frame_layout.addWidget(value_widget, i+1, 2, Qt.AlignmentFlag.AlignLeft, 2)

        layout.addWidget(frame)

        # Attach CSS-styles
        self.setStyleSheet(self.theme_helper.get_css('about_popup'))

    def eventFilter(self, obj, event):
        # Close the dialog when clicking outside it
        if (event.type() == QEvent.Type.MouseButtonPress
                or event.type() == QEvent.Type.Leave
                or event.type() == QEvent.Type.FocusOut):
            self.close()
        return super().eventFilter(obj, event)



# Hindi lexemes common.py
lexemes = {
    # Common
    "module_ondevice_llm_name": "डिवाइस पर LLM",

    "module_ondevice_llm_model_exception": "मॉडल नहीं मिला त्रुटि: {error_msg}",
    "module_ondevice_llm_task_exception": "मॉडल जेनरेटर प्रारंभ करने में असमर्थ: {error_msg}",
}



# Georgian lexemes ai_assistant.py
lexemes = {
    # ხელმძღვანელობა AI-თან დიალოგი
    "dialog_title": "მოთხოვნა AI-სთან",
    "dialog_prompt_input_placeholder_text": "მოთხოვნა",
    "dialog_prompt_input_accessible_description": "შეიყვანეთ მოთხოვნა, რათა მიეცით კონტექსტი AI ასისტენტს",
    "dialog_response_output_placeholder_text": "პასუხის მონაცემები",
    "dialog_response_output_accessible_description": "პასუხის მონაცემები გამოჩნდება ამ ველში",
    "dialog_response_output_notice_empty_text": "შეიყვანეთ ტექსტი, რათა მიიღოთ პასუხი დაახლოებით...",

    "dialog_usage_module_label": "მოდული",
    "dialog_usage_model_label": "მოდელი",
    "dialog_usage_tokens_label": "ტოკენები",
    "dialog_usage_tokens_prompt": "მოთხოვნა: {tokens}",
    "dialog_usage_tokens_answer": "პასუხი: {tokens}",
    "dialog_usage_tokens_total": "ჯამში: {tokens}",

    "dialog_button_send_request": "მოთხოვნის გაგზავნა",
    "dialog_button_stop_request": "შეჩერე",
    "dialog_button_save_history": "დიალოგის ისტორიის შენახვა",

    "dialog_message_copy_tooltip": "ტექსტის კოპირება",
    "dialog_message_copied_tooltip": "დაკოპირებულია",

    "dialog_error_loading_model": "შეცდომა საბაჟო მოდელის ჩატვირთვისას",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Contains unit and integration tests for the related functionality.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from notolog.settings import Settings
from notolog.app_config import AppConfig

from . import test_core_app  # noqa: F401

import os
import pytest


class TestSettings:

    @pytest.fixture(scope="function")
    def test_obj_settings(self, test_core_app):  # noqa: F811 redefinition of unused 'test_app'
        # The test_app fixture sets up the app environment explicitly.
        settings = Settings()
        # Clear settings to be sure start over without side effects
        settings.clear()
        # Reset singleton (qa functionality)
        Settings.reload()

        yield settings

    def test_settings_defaults(self, test_obj_settings):
        assert test_obj_settings.ui_width == 0
        assert test_obj_settings.ui_height == 0
        assert test_obj_settings.ui_pos_x == 0
        assert test_obj_settings.ui_pos_y == 0
        assert test_obj_settings.file_path == ''
        assert test_obj_settings.toolbar_icons == 0

    def test_settings_setter_getter(self, test_obj_settings):
        # Reset settings to test
        test_obj_settings.clear()

        # Set values in A object
        test_obj_settings.ui_width = 11
        test_obj_settings.ui_height = 22
        test_obj_settings.ui_pos_x = 33
        test_obj_settings.ui_pos_y = 44
        test_obj_settings.file_path = os.path.normpath('test/path/file.txt')
        test_obj_settings.toolbar_icons = 8

        # Sync changes
        test_obj_settings.sync()

        # Get fresh settings instance
        settings = Settings()

        # Assert values are stored correctly
        assert settings.ui_width == 11
        assert settings.ui_height == 22
        assert settings.ui_pos_x == 33
        assert settings.ui_pos_y == 44
        assert settings.file_path == os.path.normpath('test/path/file.txt')
        assert settings.toolbar_icons == 8

        # Linux: /home/runner/.config/Notolog/notolog_editor_tests.conf (notolog_editor_qa.conf)
        # macOS: '/Users/runner/Library/Preferences/com.notolog.notolog_editor_tests.plist'
        # Windows: '\\HKEY_CURRENT_USER\\Software\\Notolog\\notolog_editor_tests'
        assert AppConfig().get_settings_app_name_qa() in settings.fileName()
        assert test_obj_settings.fileName() == settings.fileName()

        # Clear all settings to reset to default state
        settings.clear()

        # Sync changes
        settings.sync()

        # Assert values were restored correctly
        assert settings.ui_width == 0
        assert settings.ui_height == 0
        assert settings.ui_pos_x == 0
        assert settings.ui_pos_y == 0
        assert settings.file_path == ''
        assert settings.toolbar_icons == 0

    def test_settings_clear(self, test_obj_settings: Settings):
        # Verify the test file's suffix
        settings_file_path = test_obj_settings.get_filename()
        assert '_qa' in settings_file_path
        # Clear the settings
        test_obj_settings.clear()
        assert not os.path.exists(settings_file_path)



# Portuguese lexemes settings_dialog.py
lexemes = {
    "tab_openai_api_config": "API da OpenAI",

    "module_openai_api_label": "API da OpenAI",
    "module_openai_api_url_label": "URL da API",
    "module_openai_api_url_input_placeholder_text": "URL da API",
    "module_openai_api_url_input_accessible_description":
        "A URL da API da OpenAI é o endereço do endpoint da API, que pode variar dependendo do serviço e da versão.\n"
        "O Assistente de IA usa aquele para funcionalidades de chat conversacional ou completamentos de texto.\n"
        "Consulte a documentação oficial da API da OpenAI para obter a URL atual.",
    "module_openai_api_key_label": "Chave API",
    "module_openai_api_key_input_placeholder_text": "Chave API",
    "module_openai_api_key_input_accessible_description":
        "A chave API da OpenAI é um token secreto usado para autenticar solicitações ao endpoint da API.",
    "module_openai_api_supported_models_label": "Modelos suportados",
    "module_openai_api_model_names_combo_placeholder_text": "Escolha um modelo",
    "module_openai_api_model_names_combo_accessible_description":
        "Selecione entre os modelos suportados para conversas por chat.",

    "module_openai_api_base_system_prompt_label": "Prompt do sistema",
    "module_openai_api_base_system_prompt_edit_placeholder_text":
        "Prompt de sistema base que antecede cada solicitação",
    "module_openai_api_base_system_prompt_edit_accessible_description":
        "Um prompt de sistema base que antecede cada solicitação.\n"
        "Normalmente é um texto simples com instruções ou características de papel.",

    "module_openai_api_base_response_temperature_label": "Temperatura: {temperature}",
    "module_openai_api_base_response_temperature_input_accessible_description":
        "Ajusta a aleatoriedade das respostas do modelo. Valores mais altos produzem saídas mais variadas, "
        "enquanto valores mais baixos tornam as respostas mais previsíveis.",

    "module_openai_api_base_response_max_tokens_label": "Máximo de tokens por resposta",
    "module_openai_api_base_response_max_tokens_input_accessible_description":
        "O número máximo de tokens que podem ser recebidos em uma resposta, incluindo palavras e pontuação, "
        "controlando o comprimento da saída.",

    "module_openai_api_config_prompt_history_size_label": "Tamanho do histórico de prompts",
    "module_openai_api_config_prompt_history_size_input_accessible_description":
        "Controla o número de entradas no histórico de prompts que o sistema retém para referência.\n"
        "Um valor zero permite entradas ilimitadas."
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Part of the default module.
- Functionality: Facilitates access to the module's API functions.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

import logging

from ..base_core import BaseCore

from .. import Settings


class ModuleCore(BaseCore):

    module_name = 'Default Module'

    # Functionality extended by the module.
    extensions = []

    def __init__(self, parent=None):
        super().__init__()

        self.parent = parent

        self.settings = Settings(parent=self)

        self.logger = logging.getLogger('default_module')

        self.logger.debug(f'Module {__name__} loaded')

    @staticmethod
    def get_lexemes_path():
        return None



# Latin lexemes common.py
lexemes = {
    "module_llama_cpp_name": "Modulus llama.cpp",

    "module_llama_cpp_model_exception": "Error: Model non inventus: {error_msg}",
    "module_llama_cpp_task_exception": "Generatorem modeli initiare non potest: {error_msg}",
}



# Portuguese lexemes settings_dialog.py
lexemes = {
    "tab_ondevice_llm_config": "LLM no Dispositivo",

    "module_ondevice_llm_config_label": "Modelo LLM no Dispositivo",
    "module_ondevice_llm_config_path_label": "Localização do Modelo ONNX",
    "module_ondevice_llm_config_path_input_placeholder_text": "Caminho para o diretório do modelo",
    "module_ondevice_llm_config_path_input_accessible_description":
        "Um campo de entrada com um seletor para especificar o caminho do diretório do modelo onde os arquivos\n"
        "ONNX estão localizados. Os modelos suportados estão no formato ONNX, que representa Open Neural Network Exchange,\n"
        "um padrão aberto para formatos de modelos de aprendizado de máquina.",

    "module_ondevice_llm_config_response_temperature_label": "Temperatura: {temperature}",
    "module_ondevice_llm_config_response_temperature_input_accessible_description":
        "Ajusta a aleatoriedade das respostas do modelo. Valores mais altos produzem saídas mais variadas,\n"
        "enquanto valores mais baixos tornam as respostas mais previsíveis.",

    "module_ondevice_llm_config_response_max_tokens_label": "Máximo de Tokens de Resposta",
    "module_ondevice_llm_config_response_max_tokens_input_accessible_description":
        "Define o número máximo de tokens a serem recebidos em uma resposta, incluindo palavras e pontuações,\n"
        "controlando o comprimento da saída.",

    "module_ondevice_llm_config_prompt_history_size_label": "Tamanho do Histórico de Prompts",
    "module_ondevice_llm_config_prompt_history_size_input_accessible_description":
        "Controla o número de entradas no histórico de prompts que o sistema retém para referência.\n"
        "Um valor zero permite entradas ilimitadas."
}
